@@ -1,20 +1,7 @@
 /*
- *      This file is part of GPaste.
+ * This file is part of GPaste.
  *
- *      Copyright 2014-2015 Marc-Antoine Perennou <Marc-Antoine@Perennou.com>
- *
- *      GPaste is free software: you can redistribute it and/or modify
- *      it under the terms of the GNU General Public License as published by
- *      the Free Software Foundation, either version 3 of the License, or
- *      (at your option) any later version.
- *
- *      GPaste is distributed in the hope that it will be useful,
- *      but WITHOUT ANY WARRANTY; without even the implied warranty of
- *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *      GNU General Public License for more details.
- *
- *      You should have received a copy of the GNU General Public License
- *      along with GPaste.  If not, see <http://www.gnu.org/licenses/>.
+ * Copyright (c) 2010-2016, Marc-Antoine Perennou <Marc-Antoine@Perennou.com>
  */
 /* -*- mode: js2; js2-basic-offset: 4; indent-tabs-mode: nil -*- */
 
@@ -83,7 +70,6 @@ const GPasteIndicator = new Lang.Class({
         GPaste.Client.new(Lang.bind(this, function (obj, result) {
             this._client = GPaste.Client.new_finish(result);
 
-            this._dummyHistoryItem.update();
             this._uiItem = new UiItem.GPasteUiItem(this.menu);
             this._emptyHistoryItem = new EmptyHistoryItem.GPasteEmptyHistoryItem(this._client);
             this._aboutItem = new AboutItem.GPasteAboutItem(this._client, this.menu);
@@ -156,12 +142,19 @@ const GPasteIndicator = new Lang.Class({
                 let results = this._searchResults.length;
                 let maxSize = this._history.length;
 
-                if (results > maxSize)
-                    results = maxSize;
+                if (results) {
+                    if (results > maxSize)
+                        results = maxSize;
 
-                for (let i = 0; i < results; ++i) {
-                    this._history[i].setIndex(this._searchResults[i]);
+                    for (let i = 0; i < results; ++i) {
+                        this._history[i].setIndex(this._searchResults[i]);
+                    }
+
+                    this._dummyHistoryItem.actor.hide();
+                } else {
+                  this._dummyHistoryItem.showNoResult();
                 }
+
                 for (let i = results; i < maxSize; ++i) {
                     this._history[i].setIndex(-1);
                 }
@@ -247,7 +240,7 @@ const GPasteIndicator = new Lang.Class({
 
     _updateVisibility: function(empty) {
         if (empty) {
-            this._dummyHistoryItem.actor.show();
+            this._dummyHistoryItem.showEmpty();
             this._emptyHistoryItem.actor.hide();
             this._searchItem.actor.hide();
         } else {
