@@ -1,20 +1,7 @@
 /*
- *      This file is part of GPaste.
+ * This file is part of GPaste.
  *
- *      Copyright 2014-2015 Marc-Antoine Perennou <Marc-Antoine@Perennou.com>
- *
- *      GPaste is free software: you can redistribute it and/or modify
- *      it under the terms of the GNU General Public License as published by
- *      the Free Software Foundation, either version 3 of the License, or
- *      (at your option) any later version.
- *
- *      GPaste is distributed in the hope that it will be useful,
- *      but WITHOUT ANY WARRANTY; without even the implied warranty of
- *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *      GNU General Public License for more details.
- *
- *      You should have received a copy of the GNU General Public License
- *      along with GPaste.  If not, see <http://www.gnu.org/licenses/>.
+ * Copyright (c) 2010-2016, Marc-Antoine Perennou <Marc-Antoine@Perennou.com>
  */
 
 #include <gpaste-applet-menu.h>
@@ -25,15 +12,22 @@ struct _GPasteAppletStatusIcon
     GPasteAppletIcon parent_instance;
 };
 
+enum
+{
+    C_PRESS,
+
+    C_LAST_SIGNAL
+};
+
 typedef struct
 {
     GtkStatusIcon *icon;
     GtkMenu       *menu;
 
-    gulong         press_id;
+    guint64        c_signals[C_LAST_SIGNAL];
 } GPasteAppletStatusIconPrivate;
 
-G_DEFINE_TYPE_WITH_PRIVATE (GPasteAppletStatusIcon, g_paste_applet_status_icon, G_PASTE_TYPE_APPLET_ICON)
+G_PASTE_DEFINE_TYPE_WITH_PRIVATE (AppletStatusIcon, applet_status_icon, G_PASTE_TYPE_APPLET_ICON)
 
 static gboolean
 g_paste_applet_status_icon_popup (GtkStatusIcon  *icon,
@@ -57,11 +51,11 @@ g_paste_applet_status_icon_popup (GtkStatusIcon  *icon,
 static void
 g_paste_applet_status_icon_dispose (GObject *object)
 {
-    GPasteAppletStatusIconPrivate *priv = g_paste_applet_status_icon_get_instance_private (G_PASTE_APPLET_STATUS_ICON (object));
+    const GPasteAppletStatusIconPrivate *priv = _g_paste_applet_status_icon_get_instance_private (G_PASTE_APPLET_STATUS_ICON (object));
 
     if (priv->icon)
     {
-        g_signal_handler_disconnect (priv->icon, priv->press_id);
+        g_signal_handler_disconnect (priv->icon, priv->c_signals[C_PRESS]);
         g_clear_object (&priv->icon);
     }
 
@@ -85,10 +79,10 @@ g_paste_applet_status_icon_init (GPasteAppletStatusIcon *self)
     gtk_status_icon_set_visible (priv->icon, TRUE);
     G_GNUC_END_IGNORE_DEPRECATIONS
 
-    priv->press_id = g_signal_connect (priv->icon,
-                                       "button-press-event",
-                                       G_CALLBACK (g_paste_applet_status_icon_popup),
-                                       priv);
+    priv->c_signals[C_PRESS] = g_signal_connect (priv->icon,
+                                                 "button-press-event",
+                                                 G_CALLBACK (g_paste_applet_status_icon_popup),
+                                                 priv);
 }
 
 /**
@@ -105,7 +99,7 @@ G_PASTE_VISIBLE GPasteAppletIcon *
 g_paste_applet_status_icon_new (GPasteClient *client,
                                 GApplication *app)
 {
-    g_return_val_if_fail (G_PASTE_IS_CLIENT (client), NULL);
+    g_return_val_if_fail (_G_PASTE_IS_CLIENT (client), NULL);
     g_return_val_if_fail (!app || G_IS_APPLICATION (app), NULL);
 
     GPasteAppletIcon *self = g_paste_applet_icon_new (G_PASTE_TYPE_APPLET_STATUS_ICON, client);
