@@ -1,20 +1,7 @@
 /*
- *      This file is part of GPaste.
+ * This file is part of GPaste.
  *
- *      Copyright 2011-2015 Marc-Antoine Perennou <Marc-Antoine@Perennou.com>
- *
- *      GPaste is free software: you can redistribute it and/or modify
- *      it under the terms of the GNU General Public License as published by
- *      the Free Software Foundation, either version 3 of the License, or
- *      (at your option) any later version.
- *
- *      GPaste is distributed in the hope that it will be useful,
- *      but WITHOUT ANY WARRANTY; without even the implied warranty of
- *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *      GNU General Public License for more details.
- *
- *      You should have received a copy of the GNU General Public License
- *      along with GPaste.  If not, see <http://www.gnu.org/licenses/>.
+ * Copyright (c) 2010-2016, Marc-Antoine Perennou <Marc-Antoine@Perennou.com>
  */
 
 #include <gpaste-clipboard.h>
@@ -29,6 +16,13 @@ struct _GPasteClipboard
     GObject parent_instance;
 };
 
+enum
+{
+    C_OWNER_CHANGE,
+
+    C_LAST_SIGNAL
+};
+
 typedef enum
 {
     CLIP_STATE_IDLE,
@@ -44,7 +38,7 @@ typedef struct
     gchar          *text;
     gchar          *image_checksum;
 
-    gulong          owner_change_signal;
+    guint64         c_signals[C_LAST_SIGNAL];
 
     /* GtkClipboard has a bug where it overrides an internal GMainLoop
      * if we call gtk_clipboard_store twice in less than 10s, resulting
@@ -52,7 +46,7 @@ typedef struct
     ClipState       state;
 } GPasteClipboardPrivate;
 
-G_DEFINE_TYPE_WITH_PRIVATE (GPasteClipboard, g_paste_clipboard, G_TYPE_OBJECT)
+G_PASTE_DEFINE_TYPE_WITH_PRIVATE (Clipboard, clipboard, G_TYPE_OBJECT)
 
 enum
 {
@@ -67,18 +61,13 @@ static void
 g_paste_clipboard_bootstrap_finish (GPasteClipboard *self,
                                     GPasteHistory   *history)
 {
-    GPasteClipboardPrivate *priv = g_paste_clipboard_get_instance_private (self);
+    const GPasteClipboardPrivate *priv = _g_paste_clipboard_get_instance_private (self);
 
     if (!priv->text && !priv->image_checksum)
     {
-        const GSList *h = g_paste_history_get_history (history);
+        const GList *h = g_paste_history_get_history (history);
         if (h)
-        {
-            GPasteItem *item = h->data;
-
-            if (!g_paste_clipboard_select_item (self, item))
-                g_paste_history_remove (history, 0);
-        }
+            g_paste_clipboard_select_item (self, h->data);
     }
 }
 
@@ -104,17 +93,15 @@ g_paste_clipboard_bootstrap_finish_image (GPasteClipboard *self,
  * @history: a #GPasteHistory instance
  *
  * Bootstrap a #GPasteClipboard with an initial value
- *
- * Returns:
  */
 G_PASTE_VISIBLE void
 g_paste_clipboard_bootstrap (GPasteClipboard *self,
                              GPasteHistory   *history)
 {
-    g_return_if_fail (G_PASTE_IS_CLIPBOARD (self));
-    g_return_if_fail (G_PASTE_IS_HISTORY (history));
+    g_return_if_fail (_G_PASTE_IS_CLIPBOARD (self));
+    g_return_if_fail (_G_PASTE_IS_HISTORY (history));
 
-    GPasteClipboardPrivate *priv = g_paste_clipboard_get_instance_private (self);
+    const GPasteClipboardPrivate *priv = _g_paste_clipboard_get_instance_private (self);
     GtkClipboard *real = priv->real;
 
     if (gtk_clipboard_wait_is_uris_available (real) ||
@@ -147,9 +134,9 @@ g_paste_clipboard_bootstrap (GPasteClipboard *self,
 G_PASTE_VISIBLE GdkAtom
 g_paste_clipboard_get_target (const GPasteClipboard *self)
 {
-    g_return_val_if_fail (G_PASTE_IS_CLIPBOARD (self), NULL);
+    g_return_val_if_fail (_G_PASTE_IS_CLIPBOARD (self), NULL);
 
-    GPasteClipboardPrivate *priv = g_paste_clipboard_get_instance_private (self);
+    const GPasteClipboardPrivate *priv = _g_paste_clipboard_get_instance_private (self);
 
     return priv->target;
 }
@@ -165,9 +152,9 @@ g_paste_clipboard_get_target (const GPasteClipboard *self)
 G_PASTE_VISIBLE GtkClipboard *
 g_paste_clipboard_get_real (const GPasteClipboard *self)
 {
-    g_return_val_if_fail (G_PASTE_IS_CLIPBOARD (self), NULL);
+    g_return_val_if_fail (_G_PASTE_IS_CLIPBOARD (self), NULL);
 
-    GPasteClipboardPrivate *priv = g_paste_clipboard_get_instance_private (self);
+    const GPasteClipboardPrivate *priv = _g_paste_clipboard_get_instance_private (self);
 
     return priv->real;
 }
@@ -183,24 +170,13 @@ g_paste_clipboard_get_real (const GPasteClipboard *self)
 G_PASTE_VISIBLE const gchar *
 g_paste_clipboard_get_text (const GPasteClipboard *self)
 {
-    g_return_val_if_fail (G_PASTE_IS_CLIPBOARD (self), NULL);
+    g_return_val_if_fail (_G_PASTE_IS_CLIPBOARD (self), NULL);
 
-    GPasteClipboardPrivate *priv = g_paste_clipboard_get_instance_private (self);
+    const GPasteClipboardPrivate *priv = _g_paste_clipboard_get_instance_private (self);
 
     return priv->text;
 }
 
-static const gchar *
-_g_paste_clipboard_private_target_name (const GPasteClipboardPrivate *priv)
-{
-    if (priv->target == GDK_SELECTION_CLIPBOARD)
-        return "CLIPBOARD";
-    else if (priv->target == GDK_SELECTION_PRIMARY)
-        return "PRIMARY";
-    else
-        return "UNKNOWN";
-}
-
 static void
 g_paste_clipboard_private_set_text (GPasteClipboardPrivate *priv,
                                     const gchar            *text)
@@ -208,7 +184,7 @@ g_paste_clipboard_private_set_text (GPasteClipboardPrivate *priv,
     g_free (priv->text);
     g_free (priv->image_checksum);
 
-    g_debug("%s: set text", _g_paste_clipboard_private_target_name (priv));
+    g_debug("%s: set text", gdk_atom_name (priv->target));
 
     priv->text = g_strdup (text);
     priv->image_checksum = NULL;
@@ -250,7 +226,7 @@ g_paste_clipboard_on_text_ready (GtkClipboard *clipboard G_GNUC_UNUSED,
             data->callback (self, NULL, data->user_data);
         return;
     }
-    if (priv->text && !g_strcmp0 (priv->text, to_add))
+    if (priv->text && g_paste_str_equal (priv->text, to_add))
     {
         if (data->callback)
             data->callback (self, NULL, data->user_data);
@@ -259,7 +235,7 @@ g_paste_clipboard_on_text_ready (GtkClipboard *clipboard G_GNUC_UNUSED,
 
     if (trim_items &&
         priv->target == GDK_SELECTION_CLIPBOARD &&
-        g_strcmp0 (text, stripped))
+        !g_paste_str_equal (text, stripped))
             g_paste_clipboard_select_text (self, stripped);
     else
         g_paste_clipboard_private_set_text (priv, to_add);
@@ -275,17 +251,15 @@ g_paste_clipboard_on_text_ready (GtkClipboard *clipboard G_GNUC_UNUSED,
  * @user_data: user data to pass to @callback
  *
  * Put the text from the intern GtkClipboard in the #GPasteClipboard
- *
- * Returns:
  */
 G_PASTE_VISIBLE void
 g_paste_clipboard_set_text (GPasteClipboard            *self,
                             GPasteClipboardTextCallback callback,
                             gpointer                    user_data)
 {
-    g_return_if_fail (G_PASTE_IS_CLIPBOARD (self));
+    g_return_if_fail (_G_PASTE_IS_CLIPBOARD (self));
 
-    GPasteClipboardPrivate *priv = g_paste_clipboard_get_instance_private (self);
+    const GPasteClipboardPrivate *priv = _g_paste_clipboard_get_instance_private (self);
     GPasteClipboardTextCallbackData *data = g_new (GPasteClipboardTextCallbackData, 1);
 
     data->self = self;
@@ -318,7 +292,7 @@ g_paste_clipboard_private_store_timeout (gpointer user_data)
 static void
 g_paste_clipboard_private_store (GPasteClipboardPrivate *priv)
 {
-    g_debug("%s: store", _g_paste_clipboard_private_target_name (priv));
+    g_debug("%s: store", gdk_atom_name (priv->target));
 
     switch (priv->state)
     {
@@ -344,20 +318,18 @@ g_paste_clipboard_private_store (GPasteClipboardPrivate *priv)
  * @text: the text to select
  *
  * Put the text into the #GPasteClipbaord and the intern GtkClipboard
- *
- * Returns:
  */
 G_PASTE_VISIBLE void
 g_paste_clipboard_select_text (GPasteClipboard *self,
                                const gchar     *text)
 {
-    g_return_if_fail (G_PASTE_IS_CLIPBOARD (self));
+    g_return_if_fail (_G_PASTE_IS_CLIPBOARD (self));
     g_return_if_fail (text);
     g_return_if_fail (g_utf8_validate (text, -1, NULL));
 
     GPasteClipboardPrivate *priv = g_paste_clipboard_get_instance_private (self);
 
-    g_debug("%s: select text", _g_paste_clipboard_private_target_name (priv));
+    g_debug("%s: select text", gdk_atom_name (priv->target));
 
     /* Avoid cycling twice as gtk_clipboard_set_text will make the clipboards manager react */
     g_paste_clipboard_private_set_text (priv, text);
@@ -371,7 +343,7 @@ g_paste_clipboard_get_clipboard_data (GtkClipboard     *clipboard G_GNUC_UNUSED,
                                       guint32           info      G_GNUC_UNUSED,
                                       gpointer          user_data_or_owner)
 {
-    g_return_if_fail (G_PASTE_IS_ITEM (user_data_or_owner));
+    g_return_if_fail (_G_PASTE_IS_ITEM (user_data_or_owner));
 
     GPasteItem *item = G_PASTE_ITEM (user_data_or_owner);
 
@@ -380,7 +352,7 @@ g_paste_clipboard_get_clipboard_data (GtkClipboard     *clipboard G_GNUC_UNUSED,
     /* The content is requested as text */
     if (gtk_targets_include_text (targets, 1))
         gtk_selection_data_set_text (selection_data, g_paste_item_get_real_value (item), -1);
-    else if (G_PASTE_IS_IMAGE_ITEM (item))
+    else if (_G_PASTE_IS_IMAGE_ITEM (item))
     {
         if (gtk_targets_include_image (targets, 1, TRUE))
             gtk_selection_data_set_pixbuf (selection_data, g_paste_image_item_get_image (G_PASTE_IMAGE_ITEM (item)));
@@ -388,7 +360,7 @@ g_paste_clipboard_get_clipboard_data (GtkClipboard     *clipboard G_GNUC_UNUSED,
     /* The content is requested as uris */
     else
     {
-        g_return_if_fail (G_PASTE_IS_URIS_ITEM (item));
+        g_return_if_fail (_G_PASTE_IS_URIS_ITEM (item));
 
         const gchar * const *uris = g_paste_uris_item_get_uris (G_PASTE_URIS_ITEM (item));
 
@@ -427,9 +399,9 @@ g_paste_clipboard_private_select_uris (GPasteClipboardPrivate *priv,
     GtkClipboard *real = priv->real;
     g_autoptr (GtkTargetList) target_list = gtk_target_list_new (NULL, 0);
 
-    g_debug("%s: select uris", _g_paste_clipboard_private_target_name (priv));
+    g_debug("%s: select uris", gdk_atom_name (priv->target));
 
-    g_paste_clipboard_private_set_text (priv, g_paste_item_get_real_value (G_PASTE_ITEM (item)));
+    g_paste_clipboard_private_set_text (priv, g_paste_item_get_real_value (_G_PASTE_ITEM (item)));
 
     gtk_target_list_add_text_targets (target_list, 0);
     gtk_target_list_add_uri_targets (target_list, 0);
@@ -453,13 +425,11 @@ g_paste_clipboard_private_select_uris (GPasteClipboardPrivate *priv,
  * @self: a #GPasteClipboard instance
  *
  * Clears the content of the clipboard
- *
- * Returns:
  */
 G_PASTE_VISIBLE void
 g_paste_clipboard_clear (GPasteClipboard *self)
 {
-    g_return_if_fail (G_PASTE_IS_CLIPBOARD (self));
+    g_return_if_fail (_G_PASTE_IS_CLIPBOARD (self));
 
     GPasteClipboardPrivate *priv = g_paste_clipboard_get_instance_private (self);
 
@@ -467,7 +437,7 @@ g_paste_clipboard_clear (GPasteClipboard *self)
     if (!priv->text && !priv->image_checksum)
         return;
 
-    g_debug("%s: clear", _g_paste_clipboard_private_target_name (priv));
+    g_debug("%s: clear", gdk_atom_name (priv->target));
 
     g_clear_pointer (&priv->text, g_free);
     g_clear_pointer (&priv->image_checksum, g_free);
@@ -486,9 +456,9 @@ g_paste_clipboard_clear (GPasteClipboard *self)
 G_PASTE_VISIBLE const gchar *
 g_paste_clipboard_get_image_checksum (const GPasteClipboard *self)
 {
-    g_return_val_if_fail (G_PASTE_IS_CLIPBOARD (self), NULL);
+    g_return_val_if_fail (_G_PASTE_IS_CLIPBOARD (self), NULL);
 
-    GPasteClipboardPrivate *priv = g_paste_clipboard_get_instance_private (self);
+    const GPasteClipboardPrivate *priv = _g_paste_clipboard_get_instance_private (self);
 
     return priv->image_checksum;
 }
@@ -513,7 +483,7 @@ g_paste_clipboard_private_select_image (GPasteClipboardPrivate *priv,
 
     GtkClipboard *real = priv->real;
 
-    g_debug("%s: select image", _g_paste_clipboard_private_target_name (priv));
+    g_debug("%s: select image", gdk_atom_name (priv->target));
 
     g_paste_clipboard_private_set_image_checksum (priv, checksum);
     gtk_clipboard_set_image (real, image);
@@ -546,7 +516,7 @@ g_paste_clipboard_on_image_ready (GtkClipboard *clipboard G_GNUC_UNUSED,
                                                                         (guchar *) gdk_pixbuf_get_pixels (image),
                                                                         -1);
 
-    if (g_strcmp0 (checksum, priv->image_checksum))
+    if (!g_paste_str_equal (checksum, priv->image_checksum))
     {
         g_paste_clipboard_private_select_image (priv,
                                                 image,
@@ -568,17 +538,15 @@ g_paste_clipboard_on_image_ready (GtkClipboard *clipboard G_GNUC_UNUSED,
  * @user_data: user data to pass to @callback
  *
  * Put the image from the intern GtkClipboard in the #GPasteClipboard
- *
- * Returns:
  */
 G_PASTE_VISIBLE void
 g_paste_clipboard_set_image (GPasteClipboard             *self,
                              GPasteClipboardImageCallback callback,
                              gpointer                     user_data)
 {
-    g_return_if_fail (G_PASTE_IS_CLIPBOARD (self));
+    g_return_if_fail (_G_PASTE_IS_CLIPBOARD (self));
 
-    GPasteClipboardPrivate *priv = g_paste_clipboard_get_instance_private (self);
+    const GPasteClipboardPrivate *priv = _g_paste_clipboard_get_instance_private (self);
     GPasteClipboardImageCallbackData *data = g_new (GPasteClipboardImageCallbackData, 1);
 
     data->self = self;
@@ -596,47 +564,44 @@ g_paste_clipboard_set_image (GPasteClipboard             *self,
  * @item: the item to select
  *
  * Put the value of the item into the #GPasteClipbaord and the intern GtkClipboard
- *
- * Returns: %FALSE if the item was invalid, %TRUE otherwise
  */
-G_PASTE_VISIBLE gboolean
-g_paste_clipboard_select_item (GPasteClipboard  *self,
-                               const GPasteItem *item)
+G_PASTE_VISIBLE void
+g_paste_clipboard_select_item (GPasteClipboard *self,
+                               GPasteItem      *item)
 {
-    g_return_val_if_fail (G_PASTE_IS_CLIPBOARD (self), FALSE);
-    g_return_val_if_fail (G_PASTE_IS_ITEM (item), FALSE);
+    g_return_if_fail (_G_PASTE_IS_CLIPBOARD (self));
+    g_return_if_fail (_G_PASTE_IS_ITEM (item));
 
     GPasteClipboardPrivate *priv = g_paste_clipboard_get_instance_private (self);
 
-    g_debug("%s: select item", _g_paste_clipboard_private_target_name (priv));
+    g_debug("%s: select item", gdk_atom_name (priv->target));
 
-    if (G_PASTE_IS_IMAGE_ITEM (item))
+    if (_G_PASTE_IS_IMAGE_ITEM (item))
     {
-        GPasteImageItem *image_item = G_PASTE_IMAGE_ITEM (item);
+        const GPasteImageItem *image_item = _G_PASTE_IMAGE_ITEM (item);
         const gchar *checksum = g_paste_image_item_get_checksum (image_item);
-        GdkPixbuf *image = g_paste_image_item_get_image (image_item);
 
-        if (!image || !GDK_IS_PIXBUF (image))
-            return FALSE;
-        else if (g_strcmp0 (checksum, priv->image_checksum))
-            g_paste_clipboard_private_select_image (priv, image, checksum);
+        if (!g_paste_str_equal (checksum, priv->image_checksum))
+        {
+            g_paste_clipboard_private_select_image (priv,
+                                                    g_paste_image_item_get_image (image_item),
+                                                    checksum);
+        }
     }
     else
     {
         const gchar *text = g_paste_item_get_real_value (item);
 
-        if (g_strcmp0 (text, priv->text))
+        if (!g_paste_str_equal (text, priv->text))
         {
-            if (G_PASTE_IS_URIS_ITEM (item))
+            if (_G_PASTE_IS_URIS_ITEM (item))
                 g_paste_clipboard_private_select_uris (priv, G_PASTE_URIS_ITEM (item));
-            else  if (G_PASTE_IS_TEXT_ITEM (item))
+            else  if (_G_PASTE_IS_TEXT_ITEM (item))
                 g_paste_clipboard_select_text (self, text);
             else
                 g_assert_not_reached ();
         }
     }
-
-    return TRUE;
 }
 
 /**
@@ -645,25 +610,18 @@ g_paste_clipboard_select_item (GPasteClipboard  *self,
  * @history: a #GPasteHistory instance
  *
  * Ensure the clipboard has some contents (as long as the history's not empty)
- *
- * Returns:
  */
 G_PASTE_VISIBLE void
-g_paste_clipboard_ensure_not_empty (GPasteClipboard *self,
-                                    GPasteHistory   *history)
+g_paste_clipboard_ensure_not_empty (GPasteClipboard     *self,
+                                    const GPasteHistory *history)
 {
-    g_return_if_fail (G_PASTE_IS_CLIPBOARD (self));
-    g_return_if_fail (G_PASTE_IS_HISTORY (history));
+    g_return_if_fail (_G_PASTE_IS_CLIPBOARD (self));
+    g_return_if_fail (_G_PASTE_IS_HISTORY (history));
 
-    const GSList *hist = g_paste_history_get_history (history);
+    const GList *hist = g_paste_history_get_history (history);
 
     if (hist)
-    {
-        GPasteItem *item = hist->data;
-
-        if (!g_paste_clipboard_select_item (self, item))
-            g_paste_history_remove (history, 0);
-    }
+        g_paste_clipboard_select_item (self, hist->data);
 }
 
 static void
@@ -674,7 +632,7 @@ g_paste_clipboard_owner_change (GtkClipboard        *clipboard G_GNUC_UNUSED,
     GPasteClipboard *self = user_data;
     GPasteClipboardPrivate *priv = g_paste_clipboard_get_instance_private (self);
 
-    g_debug("%s: owner change", _g_paste_clipboard_private_target_name (priv));
+    g_debug("%s: owner change", gdk_atom_name (priv->target));
 
     g_signal_emit (self,
 		   signals[OWNER_CHANGE],
@@ -689,9 +647,9 @@ g_paste_clipboard_fake_event_finish_text (GtkClipboard *clipboard G_GNUC_UNUSED,
                                           gpointer      user_data)
 {
     GPasteClipboard *self = user_data;
-    GPasteClipboardPrivate *priv = g_paste_clipboard_get_instance_private (self);
+    const GPasteClipboardPrivate *priv = _g_paste_clipboard_get_instance_private (self);
 
-    if (g_strcmp0 (text, priv->text))
+    if (!g_paste_str_equal (text, priv->text))
         g_paste_clipboard_owner_change (NULL, NULL, self);
 }
 
@@ -701,10 +659,10 @@ g_paste_clipboard_fake_event_finish_image (GtkClipboard *clipboard G_GNUC_UNUSED
                                            gpointer      user_data)
 {
     GPasteClipboard *self = user_data;
-    GPasteClipboardPrivate *priv = g_paste_clipboard_get_instance_private (self);
+    const GPasteClipboardPrivate *priv = _g_paste_clipboard_get_instance_private (self);
     g_autofree gchar *checksum = g_paste_util_compute_checksum (image);
 
-    if (g_strcmp0 (checksum, priv->image_checksum))
+    if (!g_paste_str_equal (checksum, priv->image_checksum))
         g_paste_clipboard_owner_change (NULL, NULL, self);
 
     g_object_unref (image);
@@ -714,7 +672,7 @@ static gboolean
 g_paste_clipboard_fake_event (gpointer user_data)
 {
     GPasteClipboard *self = user_data;
-    GPasteClipboardPrivate *priv = g_paste_clipboard_get_instance_private (self);
+    const GPasteClipboardPrivate *priv = _g_paste_clipboard_get_instance_private (self);
 
     if (priv->text)
         gtk_clipboard_request_text (priv->real, g_paste_clipboard_fake_event_finish_text, self);
@@ -729,11 +687,11 @@ g_paste_clipboard_fake_event (gpointer user_data)
 static void
 g_paste_clipboard_dispose (GObject *object)
 {
-    GPasteClipboardPrivate *priv = g_paste_clipboard_get_instance_private (G_PASTE_CLIPBOARD (object));
+    const GPasteClipboardPrivate *priv = _g_paste_clipboard_get_instance_private (G_PASTE_CLIPBOARD (object));
 
     if (priv->settings)
     {
-        g_signal_handler_disconnect (priv->real, priv->owner_change_signal);
+        g_signal_handler_disconnect (priv->real, priv->c_signals[C_OWNER_CHANGE]);
         g_clear_object (&priv->settings);
     }
 
@@ -743,7 +701,7 @@ g_paste_clipboard_dispose (GObject *object)
 static void
 g_paste_clipboard_finalize (GObject *object)
 {
-    GPasteClipboardPrivate *priv = g_paste_clipboard_get_instance_private (G_PASTE_CLIPBOARD (object));
+    const GPasteClipboardPrivate *priv = _g_paste_clipboard_get_instance_private (G_PASTE_CLIPBOARD (object));
 
     g_free (priv->text);
     g_free (priv->image_checksum);
@@ -785,9 +743,6 @@ g_paste_clipboard_init (GPasteClipboard *self)
 {
     GPasteClipboardPrivate *priv = g_paste_clipboard_get_instance_private (self);
 
-    priv->text = NULL;
-    priv->image_checksum = NULL;
-
     priv->state = CLIP_STATE_IDLE;
 }
 
@@ -805,7 +760,7 @@ G_PASTE_VISIBLE GPasteClipboard *
 g_paste_clipboard_new (GdkAtom         target,
                        GPasteSettings *settings)
 {
-    g_return_val_if_fail (G_PASTE_IS_SETTINGS (settings), NULL);
+    g_return_val_if_fail (_G_PASTE_IS_SETTINGS (settings), NULL);
 
     GPasteClipboard *self = g_object_new (G_PASTE_TYPE_CLIPBOARD, NULL);
     GPasteClipboardPrivate *priv = g_paste_clipboard_get_instance_private (self);
@@ -815,10 +770,10 @@ g_paste_clipboard_new (GdkAtom         target,
 
     GtkClipboard *real = priv->real = gtk_clipboard_get (target);
 
-    priv->owner_change_signal = g_signal_connect (real,
-                                                  "owner-change",
-                                                  G_CALLBACK (g_paste_clipboard_owner_change),
-                                                  self);
+    priv->c_signals[C_OWNER_CHANGE] = g_signal_connect (real,
+                                                        "owner-change",
+                                                        G_CALLBACK (g_paste_clipboard_owner_change),
+                                                        self);
 
     if (!gdk_display_request_selection_notification (gdk_display_get_default (), target))
     {
