@@ -1,20 +1,7 @@
 /*
- *      This file is part of GPaste.
+ * This file is part of GPaste.
  *
- *      Copyright 2011-2015 Marc-Antoine Perennou <Marc-Antoine@Perennou.com>
- *
- *      GPaste is free software: you can redistribute it and/or modify
- *      it under the terms of the GNU General Public License as published by
- *      the Free Software Foundation, either version 3 of the License, or
- *      (at your option) any later version.
- *
- *      GPaste is distributed in the hope that it will be useful,
- *      but WITHOUT ANY WARRANTY; without even the implied warranty of
- *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *      GNU General Public License for more details.
- *
- *      You should have received a copy of the GNU General Public License
- *      along with GPaste.  If not, see <http://www.gnu.org/licenses/>.
+ * Copyright (c) 2010-2016, Marc-Antoine Perennou <Marc-Antoine@Perennou.com>
  */
 
 #include <gpaste-clipboards-manager.h>
@@ -26,22 +13,36 @@ struct _GPasteClipboardsManager
     GObject parent_instance;
 };
 
+enum
+{
+    C_CLIP_OWNER_CHANGE,
+
+    C_CLIP_LAST_SIGNAL
+};
+
 typedef struct
 {
     GPasteClipboard *clipboard;
-    gulong           signal;
+    guint64          c_signals[C_CLIP_LAST_SIGNAL];
 } _Clipboard;
 
+enum
+{
+    C_SELECTED,
+
+    C_LAST_SIGNAL
+};
+
 typedef struct
 {
     GSList         *clipboards;
     GPasteHistory  *history;
     GPasteSettings *settings;
 
-    gulong          selected_signal;
+    guint64         c_signals[C_LAST_SIGNAL];
 } GPasteClipboardsManagerPrivate;
 
-G_DEFINE_TYPE_WITH_PRIVATE (GPasteClipboardsManager, g_paste_clipboards_manager, G_TYPE_OBJECT)
+G_PASTE_DEFINE_TYPE_WITH_PRIVATE (ClipboardsManager, clipboards_manager, G_TYPE_OBJECT)
 
 /**
  * g_paste_clipboards_manager_add_clipboard:
@@ -49,15 +50,13 @@ G_DEFINE_TYPE_WITH_PRIVATE (GPasteClipboardsManager, g_paste_clipboards_manager,
  * @clipboard: (transfer none): the GPasteClipboard to add
  *
  * Add a #GPasteClipboard to the #GPasteClipboardsManager
- *
- * Returns:
  */
 G_PASTE_VISIBLE void
 g_paste_clipboards_manager_add_clipboard (GPasteClipboardsManager *self,
                                           GPasteClipboard         *clipboard)
 {
-    g_return_if_fail (G_PASTE_IS_CLIPBOARDS_MANAGER (self));
-    g_return_if_fail (G_PASTE_IS_CLIPBOARD (clipboard));
+    g_return_if_fail (_G_PASTE_IS_CLIPBOARDS_MANAGER (self));
+    g_return_if_fail (_G_PASTE_IS_CLIPBOARD (clipboard));
 
     GPasteClipboardsManagerPrivate *priv = g_paste_clipboards_manager_get_instance_private (self);
     _Clipboard *clip = g_new0 (_Clipboard, 1);
@@ -84,22 +83,18 @@ g_paste_clipboards_manager_sync_ready (GtkClipboard *clipboard G_GNUC_UNUSED,
  * @to: the destination clipboard type
  *
  * Sync a clipboard into another
- *
- * Returns:
  */
 G_PASTE_VISIBLE void
 g_paste_clipboards_manager_sync_from_to (GPasteClipboardsManager *self,
                                          GdkAtom                  from,
                                          GdkAtom                  to)
 {
-    g_return_if_fail (G_PASTE_IS_CLIPBOARDS_MANAGER (self));
+    g_return_if_fail (_G_PASTE_IS_CLIPBOARDS_MANAGER (self));
 
-    GPasteClipboardsManagerPrivate *priv = g_paste_clipboards_manager_get_instance_private (self);
+    const GPasteClipboardsManagerPrivate *priv = _g_paste_clipboards_manager_get_instance_private (self);
     GtkClipboard *_from = NULL, *_to = NULL;
-    g_autofree gchar *_from_name = gdk_atom_name (from);
-    g_autofree gchar *_to_name = gdk_atom_name (to);
 
-    g_debug ("clipboards-manager: sync from %s to %s", _from_name, _to_name);
+    g_debug ("clipboards-manager: sync from %s to %s", gdk_atom_name (from), gdk_atom_name (to));
 
     for (GSList *clipboard = priv->clipboards; clipboard; clipboard = g_slist_next (clipboard))
     {
@@ -150,7 +145,7 @@ g_paste_clipboards_manager_notify_finish (GPasteClipboardsManagerPrivate *priv,
 
             const gchar *text = g_paste_clipboard_get_text (clip);
 
-            if (!text || g_strcmp0 (text, synchronized_text))
+            if (!text || !g_paste_str_equal (text, synchronized_text))
                 g_paste_clipboard_select_text (clip, synchronized_text);
         }
     }
@@ -287,13 +282,11 @@ g_paste_clipboards_manager_notify (GPasteClipboard *clipboard,
  * @self: a #GPasteClipboardsManager instance
  *
  * Activate the #GPasteClipboardsManager
- *
- * Returns:
  */
 G_PASTE_VISIBLE void
 g_paste_clipboards_manager_activate (GPasteClipboardsManager *self)
 {
-    g_return_if_fail (G_PASTE_IS_CLIPBOARDS_MANAGER (self));
+    g_return_if_fail (_G_PASTE_IS_CLIPBOARDS_MANAGER (self));
 
     GPasteClipboardsManagerPrivate *priv = g_paste_clipboards_manager_get_instance_private (self);
 
@@ -301,10 +294,10 @@ g_paste_clipboards_manager_activate (GPasteClipboardsManager *self)
     {
         _Clipboard *clip = clipboard->data;
 
-        clip->signal = g_signal_connect (clip->clipboard,
-                                         "owner-change",
-                                         G_CALLBACK (g_paste_clipboards_manager_notify),
-                                         priv);
+        clip->c_signals[C_CLIP_OWNER_CHANGE] = g_signal_connect (clip->clipboard,
+                                                                 "owner-change",
+                                                                 G_CALLBACK (g_paste_clipboards_manager_notify),
+                                                                 priv);
     }
 }
 
@@ -314,17 +307,15 @@ g_paste_clipboards_manager_activate (GPasteClipboardsManager *self)
  * @item: the #GPasteItem to select
  *
  * Select a new #GPasteItem
- *
- * Returns: %FALSE if the item was invalid, %TRUE otherwise
  */
-G_PASTE_VISIBLE gboolean
+G_PASTE_VISIBLE void
 g_paste_clipboards_manager_select (GPasteClipboardsManager *self,
-                                   const GPasteItem        *item)
+                                   GPasteItem              *item)
 {
-    g_return_val_if_fail (G_PASTE_IS_CLIPBOARDS_MANAGER (self), FALSE);
-    g_return_val_if_fail (G_PASTE_IS_ITEM (item), FALSE);
+    g_return_if_fail (_G_PASTE_IS_CLIPBOARDS_MANAGER (self));
+    g_return_if_fail (_G_PASTE_IS_ITEM (item));
 
-    GPasteClipboardsManagerPrivate *priv = g_paste_clipboards_manager_get_instance_private (self);
+    const GPasteClipboardsManagerPrivate *priv = _g_paste_clipboards_manager_get_instance_private (self);
 
     g_debug ("clipboards-manager: select");
 
@@ -332,14 +323,8 @@ g_paste_clipboards_manager_select (GPasteClipboardsManager *self,
     {
         _Clipboard *clip = clipboard->data;
 
-        if (!g_paste_clipboard_select_item (clip->clipboard, item))
-        {
-            g_debug ("clipboards-manager: item was invalid, deleting it");
-            return FALSE;
-        }
+        g_paste_clipboard_select_item (clip->clipboard, item);
     }
-
-    return TRUE;
 }
 
 static void
@@ -347,10 +332,7 @@ on_item_selected (GPasteClipboardsManager *self,
                   GPasteItem              *item,
                   GPasteHistory           *history G_GNUC_UNUSED)
 {
-    GPasteClipboardsManagerPrivate *priv = g_paste_clipboards_manager_get_instance_private (G_PASTE_CLIPBOARDS_MANAGER (self));
-
-    if (!g_paste_clipboards_manager_select (self, item))
-        g_paste_history_remove (priv->history, 0);
+    g_paste_clipboards_manager_select (self, item);
 }
 
 static void
@@ -358,7 +340,7 @@ _clipboard_free (gpointer data)
 {
     _Clipboard *clip = data;
 
-    g_signal_handler_disconnect (clip->clipboard, clip->signal);
+    g_signal_handler_disconnect (clip->clipboard, clip->c_signals[C_CLIP_OWNER_CHANGE]);
     g_object_unref (clip->clipboard);
     g_free (clip);
 }
@@ -371,7 +353,7 @@ g_paste_clipboards_manager_dispose (GObject *object)
 
     if (settings)
     {
-        g_signal_handler_disconnect (settings, priv->selected_signal);
+        g_signal_handler_disconnect (settings, priv->c_signals[C_SELECTED]);
         g_clear_object (&priv->settings);
         g_clear_object (&priv->history);
     }
@@ -392,11 +374,8 @@ g_paste_clipboards_manager_class_init (GPasteClipboardsManagerClass *klass)
 }
 
 static void
-g_paste_clipboards_manager_init (GPasteClipboardsManager *self)
+g_paste_clipboards_manager_init (GPasteClipboardsManager *self G_GNUC_UNUSED)
 {
-    GPasteClipboardsManagerPrivate *priv = g_paste_clipboards_manager_get_instance_private (self);
-
-    priv->clipboards = NULL;
 }
 
 /**
@@ -413,8 +392,8 @@ G_PASTE_VISIBLE GPasteClipboardsManager *
 g_paste_clipboards_manager_new (GPasteHistory  *history,
                                 GPasteSettings *settings)
 {
-    g_return_val_if_fail (G_PASTE_IS_HISTORY (history), NULL);
-    g_return_val_if_fail (G_PASTE_IS_SETTINGS (settings), NULL);
+    g_return_val_if_fail (_G_PASTE_IS_HISTORY (history), NULL);
+    g_return_val_if_fail (_G_PASTE_IS_SETTINGS (settings), NULL);
 
     GPasteClipboardsManager *self = g_object_new (G_PASTE_TYPE_CLIPBOARDS_MANAGER, NULL);
     GPasteClipboardsManagerPrivate *priv = g_paste_clipboards_manager_get_instance_private (self);
@@ -422,10 +401,10 @@ g_paste_clipboards_manager_new (GPasteHistory  *history,
     priv->history = g_object_ref (history);
     priv->settings = g_object_ref (settings);
 
-    priv->selected_signal = g_signal_connect_swapped (history,
-                                                      "selected",
-                                                      G_CALLBACK (on_item_selected),
-                                                      self);
+    priv->c_signals[C_SELECTED] = g_signal_connect_swapped (history,
+                                                            "selected",
+                                                            G_CALLBACK (on_item_selected),
+                                                            self);
 
     return self;
 }
