@@ -1,25 +1,13 @@
 /*
- *      This file is part of GPaste.
+ * This file is part of GPaste.
  *
- *      Copyright 2011-2015 Marc-Antoine Perennou <Marc-Antoine@Perennou.com>
- *
- *      GPaste is free software: you can redistribute it and/or modify
- *      it under the terms of the GNU General Public License as published by
- *      the Free Software Foundation, either version 3 of the License, or
- *      (at your option) any later version.
- *
- *      GPaste is distributed in the hope that it will be useful,
- *      but WITHOUT ANY WARRANTY; without even the implied warranty of
- *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *      GNU General Public License for more details.
- *
- *      You should have received a copy of the GNU General Public License
- *      along with GPaste.  If not, see <http://www.gnu.org/licenses/>.
+ * Copyright (c) 2010-2016, Marc-Antoine Perennou <Marc-Antoine@Perennou.com>
  */
 
 #include "gpaste-gdbus-macros.h"
 
 #include <gpaste-keybinder.h>
+#include <gpaste-image-item.h>
 #include <gpaste-make-password-keybinding.h>
 #include <gpaste-pop-keybinding.h>
 #include <gpaste-screensaver-client.h>
@@ -89,10 +77,10 @@ typedef struct
     GDBusNodeInfo           *g_paste_daemon_dbus_info;
     GDBusInterfaceVTable     g_paste_daemon_dbus_vtable;
 
-    gulong                   c_signals[C_LAST_SIGNAL];
+    guint64                  c_signals[C_LAST_SIGNAL];
 } GPasteDaemonPrivate;
 
-G_DEFINE_TYPE_WITH_PRIVATE (GPasteDaemon, g_paste_daemon, G_PASTE_TYPE_BUS_OBJECT)
+G_PASTE_DEFINE_TYPE_WITH_PRIVATE (Daemon, daemon, G_PASTE_TYPE_BUS_OBJECT)
 
 enum
 {
@@ -176,7 +164,7 @@ g_paste_daemon_update (GPasteDaemon      *self,
                        GPasteUpdateTarget target,
                        guint64            position)
 {
-    GPasteDaemonPrivate *priv = g_paste_daemon_get_instance_private (self);
+    const GPasteDaemonPrivate *priv = _g_paste_daemon_get_instance_private (self);
 
     GVariant *data[] = {
         g_variant_new_string (g_enum_get_value (g_type_class_peek (G_PASTE_TYPE_UPDATE_ACTION), action)->value_nick),
@@ -192,16 +180,14 @@ g_paste_daemon_update (GPasteDaemon      *self,
  * @error: a #GError
  *
  * Emit the signal to show history
- *
- * Returns:
  */
 G_PASTE_VISIBLE void
 g_paste_daemon_show_history (GPasteDaemon *self,
                              GError      **error)
 {
-    g_return_if_fail (G_PASTE_IS_DAEMON (self));
+    g_return_if_fail (_G_PASTE_IS_DAEMON (self));
 
-    GPasteDaemonPrivate *priv = g_paste_daemon_get_instance_private (self);
+    const GPasteDaemonPrivate *priv = _g_paste_daemon_get_instance_private (self);
 
     G_PASTE_SEND_DBUS_SIGNAL_WITH_ERROR (SHOW_HISTORY);
 }
@@ -211,7 +197,7 @@ g_paste_daemon_tracking (GPasteDaemon   *self,
                          gboolean        tracking_state,
                          GPasteSettings *settings G_GNUC_UNUSED)
 {
-    GPasteDaemonPrivate *priv = g_paste_daemon_get_instance_private (self);
+    const GPasteDaemonPrivate *priv = _g_paste_daemon_get_instance_private (self);
     GVariant *variant = g_variant_new_boolean (tracking_state);
 
     G_PASTE_SEND_DBUS_SIGNAL_WITH_DATA (TRACKING, variant);
@@ -222,19 +208,18 @@ g_paste_daemon_tracking (GPasteDaemon   *self,
 /****************/
 
 static void
-g_paste_daemon_private_do_add_item (GPasteDaemonPrivate *priv,
-                                    GPasteItem          *item)
+g_paste_daemon_private_do_add_item (const GPasteDaemonPrivate *priv,
+                                    GPasteItem                *item)
 {
     g_paste_history_add (priv->history, item);
-    if (!g_paste_clipboards_manager_select (priv->clipboards_manager, item))
-        g_paste_history_remove (priv->history, 0);
+    g_paste_clipboards_manager_select (priv->clipboards_manager, item);
 }
 
 static void
-g_paste_daemon_private_do_add (GPasteDaemonPrivate *priv,
-                               const gchar         *text,
-                               guint64              length,
-                               GPasteDBusError    **err)
+g_paste_daemon_private_do_add (const GPasteDaemonPrivate *priv,
+                               const gchar               *text,
+                               guint64                    length,
+                               GPasteDBusError          **err)
 {
     G_PASTE_DBUS_ASSERT (text && length, "no content to add");
 
@@ -251,9 +236,9 @@ g_paste_daemon_private_do_add (GPasteDaemonPrivate *priv,
 }
 
 static void
-g_paste_daemon_private_add (GPasteDaemonPrivate *priv,
-                            GVariant            *parameters,
-                            GPasteDBusError    **err)
+g_paste_daemon_private_add (const GPasteDaemonPrivate *priv,
+                            GVariant                  *parameters,
+                            GPasteDBusError          **err)
 {
     guint64 length;
     g_autofree gchar *text = g_paste_daemon_get_dbus_string_parameter (parameters, &length);
@@ -262,10 +247,10 @@ g_paste_daemon_private_add (GPasteDaemonPrivate *priv,
 }
 
 static void
-g_paste_daemon_private_add_file (GPasteDaemonPrivate *priv,
-                                 GVariant            *parameters,
-                                 GError             **error,
-                                 GPasteDBusError    **err)
+g_paste_daemon_private_add_file (const GPasteDaemonPrivate *priv,
+                                 GVariant                  *parameters,
+                                 GError                   **error,
+                                 GPasteDBusError          **err)
 {
     guint64 length;
     g_autofree gchar *file = g_paste_daemon_get_dbus_string_parameter (parameters, &length);
@@ -278,14 +263,23 @@ g_paste_daemon_private_add_file (GPasteDaemonPrivate *priv,
                              &length,
                              error))
     {
-        g_paste_daemon_private_do_add (priv, content, length, err);
+        if (g_utf8_validate (content, length, NULL))
+        {
+            g_paste_daemon_private_do_add (priv, content, length, err);
+        }
+        else
+        {
+            g_autoptr (GdkPixbuf) img = gdk_pixbuf_new_from_file (file, NULL /* Error */);
+
+            g_paste_daemon_private_do_add_item (priv, g_paste_image_item_new (img));
+        }
     }
 }
 
 static void
-g_paste_daemon_private_add_password (GPasteDaemonPrivate *priv,
-                                     GVariant            *parameters,
-                                     GPasteDBusError    **err)
+g_paste_daemon_private_add_password (const GPasteDaemonPrivate *priv,
+                                     GVariant                  *parameters,
+                                     GPasteDBusError          **err)
 {
     g_autofree gchar *name = NULL;
     g_autofree gchar *password = NULL;
@@ -294,13 +288,13 @@ g_paste_daemon_private_add_password (GPasteDaemonPrivate *priv,
 
     G_PASTE_DBUS_ASSERT (name && password, "no password to add");
 
-    g_paste_daemon_private_do_add_item (priv,
-                                        g_paste_password_item_new (name, password));
+    g_paste_history_delete_password (priv->history, name);
+    g_paste_daemon_private_do_add_item (priv, g_paste_password_item_new (name, password));
 }
 
 static void
-g_paste_daemon_private_delete_history_signal (GPasteDaemonPrivate *priv,
-                                              const gchar         *history)
+g_paste_daemon_private_delete_history_signal (const GPasteDaemonPrivate *priv,
+                                              const gchar               *history)
 {
     GVariant *variant = g_variant_new_string (history);
 
@@ -308,8 +302,8 @@ g_paste_daemon_private_delete_history_signal (GPasteDaemonPrivate *priv,
 }
 
 static void
-g_paste_daemon_private_empty_history_signal (GPasteDaemonPrivate *priv,
-                                             const gchar         *history)
+g_paste_daemon_private_empty_history_signal (const GPasteDaemonPrivate *priv,
+                                             const gchar               *history)
 {
     GVariant *variant = g_variant_new_string (history);
 
@@ -317,8 +311,8 @@ g_paste_daemon_private_empty_history_signal (GPasteDaemonPrivate *priv,
 }
 
 static void
-g_paste_daemon_private_switch_history_signal (GPasteDaemonPrivate *priv,
-                                              const gchar         *history)
+g_paste_daemon_private_switch_history_signal (const GPasteDaemonPrivate *priv,
+                                              const gchar               *history)
 {
     GVariant *variant = g_variant_new_string (history);
 
@@ -326,9 +320,9 @@ g_paste_daemon_private_switch_history_signal (GPasteDaemonPrivate *priv,
 }
 
 static void
-g_paste_daemon_private_backup_history (GPasteDaemonPrivate *priv,
-                                       GVariant            *parameters,
-                                       GPasteDBusError    **err)
+g_paste_daemon_private_backup_history (const GPasteDaemonPrivate *priv,
+                                       GVariant                  *parameters,
+                                       GPasteDBusError          **err)
 {
     g_autofree gchar *history = NULL;
     g_autofree gchar *backup = NULL;
@@ -352,17 +346,17 @@ g_paste_daemon_private_backup_history (GPasteDaemonPrivate *priv,
 }
 
 static void
-g_paste_daemon_private_delete (GPasteDaemonPrivate *priv,
-                               GVariant            *parameters)
+g_paste_daemon_private_delete (const GPasteDaemonPrivate *priv,
+                               GVariant                  *parameters)
 {
     g_paste_history_remove (priv->history,
                             g_paste_daemon_get_dbus_uint64_parameter (parameters));
 }
 
 static void
-g_paste_daemon_private_delete_history (GPasteDaemonPrivate *priv,
-                                       GVariant            *parameters,
-                                       GPasteDBusError    **err)
+g_paste_daemon_private_delete_history (const GPasteDaemonPrivate *priv,
+                                       GVariant                  *parameters,
+                                       GPasteDBusError          **err)
 {
     g_autofree gchar *name = g_paste_daemon_get_dbus_string_parameter (parameters, NULL);
 
@@ -373,14 +367,14 @@ g_paste_daemon_private_delete_history (GPasteDaemonPrivate *priv,
     g_paste_history_delete (history, name, NULL);
     g_paste_daemon_private_delete_history_signal (priv, name);
 
-    if (!g_strcmp0 (name, g_paste_history_get_current (priv->history)))
-        g_paste_history_switch (history, DEFAULT_HISTORY);
+    if (g_paste_str_equal (name, g_paste_history_get_current (priv->history)))
+        g_paste_history_switch (history, G_PASTE_DEFAULT_HISTORY);
 }
 
 static void
-g_paste_daemon_private_delete_password (GPasteDaemonPrivate *priv,
-                                        GVariant            *parameters,
-                                        GPasteDBusError    **err)
+g_paste_daemon_private_delete_password (const GPasteDaemonPrivate *priv,
+                                        GVariant                  *parameters,
+                                        GPasteDBusError          **err)
 {
     g_autofree gchar *name = g_paste_daemon_get_dbus_string_parameter (parameters, NULL);
 
@@ -390,12 +384,12 @@ g_paste_daemon_private_delete_password (GPasteDaemonPrivate *priv,
 }
 
 static void
-g_paste_daemon_private_empty_history (GPasteDaemonPrivate *priv,
-                                      GVariant            *parameters)
+g_paste_daemon_private_empty_history (const GPasteDaemonPrivate *priv,
+                                      GVariant                  *parameters)
 {
     g_autofree gchar *name = g_paste_daemon_get_dbus_string_parameter (parameters, NULL);
 
-    if (!g_strcmp0 (name, g_paste_history_get_current (priv->history)))
+    if (g_paste_str_equal (name, g_paste_history_get_current (priv->history)))
     {
         g_paste_history_empty (priv->history);
     }
@@ -410,9 +404,9 @@ g_paste_daemon_private_empty_history (GPasteDaemonPrivate *priv,
 }
 
 static GVariant *
-g_paste_daemon_private_get_element (GPasteDaemonPrivate *priv,
-                                    GVariant            *parameters,
-                                    GPasteDBusError    **err)
+g_paste_daemon_private_get_element (const GPasteDaemonPrivate *priv,
+                                    GVariant                  *parameters,
+                                    GPasteDBusError          **err)
 {
     GPasteHistory *history = priv->history;
     guint64 index = g_paste_daemon_get_dbus_uint64_parameter (parameters);
@@ -429,9 +423,9 @@ g_paste_daemon_private_get_element (GPasteDaemonPrivate *priv,
 }
 
 static GVariant *
-g_paste_daemon_private_get_element_kind (GPasteDaemonPrivate *priv,
-                                         GVariant            *parameters,
-                                         GPasteDBusError    **err)
+g_paste_daemon_private_get_element_kind (const GPasteDaemonPrivate *priv,
+                                         GVariant                  *parameters,
+                                         GPasteDBusError          **err)
 {
     GPasteHistory *history = priv->history;
     guint64 index = g_paste_daemon_get_dbus_uint64_parameter (parameters);
@@ -448,9 +442,9 @@ g_paste_daemon_private_get_element_kind (GPasteDaemonPrivate *priv,
 }
 
 static GVariant *
-g_paste_daemon_private_get_elements (GPasteDaemonPrivate *priv,
-                                     GVariant            *parameters,
-                                     GPasteDBusError    **err)
+g_paste_daemon_private_get_elements (const GPasteDaemonPrivate *priv,
+                                     GVariant                  *parameters,
+                                     GPasteDBusError          **err)
 {
     GPasteHistory *history = priv->history;
     GVariantIter parameters_iter;
@@ -477,13 +471,13 @@ g_paste_daemon_private_get_elements (GPasteDaemonPrivate *priv,
 }
 
 static GVariant *
-g_paste_daemon_private_get_history (GPasteDaemonPrivate *priv)
+g_paste_daemon_private_get_history (const GPasteDaemonPrivate *priv)
 {
-    const GSList *history = g_paste_history_get_history (priv->history);
-    guint64 length = g_slist_length ((GSList *) history);
+    const GList *history = g_paste_history_get_history (priv->history);
+    guint64 length = g_list_length ((GList *) history);
     g_autofree const gchar **displayed_history = g_new (const gchar *, length + 1);
 
-    for (guint64 i = 0; i < length; ++i, history = g_slist_next (history))
+    for (guint64 i = 0; i < length; ++i, history = g_list_next (history))
         displayed_history[i] = g_paste_item_get_display_string (history->data);
     displayed_history[length] = NULL;
 
@@ -493,20 +487,20 @@ g_paste_daemon_private_get_history (GPasteDaemonPrivate *priv)
 }
 
 static GVariant *
-g_paste_daemon_private_get_history_name (GPasteDaemonPrivate *priv)
+g_paste_daemon_private_get_history_name (const GPasteDaemonPrivate *priv)
 {
     GVariant *variant = g_variant_new_string (g_paste_history_get_current (priv->history));
     return g_variant_new_tuple (&variant, 1);
 }
 
 static GVariant *
-g_paste_daemon_private_get_history_size (GPasteDaemonPrivate *priv,
-                                         GVariant            *parameters)
+g_paste_daemon_private_get_history_size (const GPasteDaemonPrivate *priv,
+                                         GVariant                  *parameters)
 {
     g_autofree gchar *name = g_paste_daemon_get_dbus_string_parameter (parameters, NULL);
     guint64 size;
 
-    if (!g_strcmp0 (name, g_paste_history_get_current (priv->history)))
+    if (g_paste_str_equal (name, g_paste_history_get_current (priv->history)))
     {
         size = g_paste_history_get_length (priv->history);
     }
@@ -524,9 +518,9 @@ g_paste_daemon_private_get_history_size (GPasteDaemonPrivate *priv,
 }
 
 static GVariant *
-g_paste_daemon_private_get_raw_element (GPasteDaemonPrivate *priv,
-                                        GVariant            *parameters,
-                                        GPasteDBusError    **err)
+g_paste_daemon_private_get_raw_element (const GPasteDaemonPrivate *priv,
+                                        GVariant                  *parameters,
+                                        GPasteDBusError          **err)
 {
     GPasteHistory *history = priv->history;
     guint64 index = g_paste_daemon_get_dbus_uint64_parameter (parameters);
@@ -543,13 +537,13 @@ g_paste_daemon_private_get_raw_element (GPasteDaemonPrivate *priv,
 }
 
 static GVariant *
-g_paste_daemon_private_get_raw_history (GPasteDaemonPrivate *priv)
+g_paste_daemon_private_get_raw_history (const GPasteDaemonPrivate *priv)
 {
-    const GSList *history = g_paste_history_get_history (priv->history);
-    guint64 length = g_slist_length ((GSList *) history);
+    const GList *history = g_paste_history_get_history (priv->history);
+    guint64 length = g_list_length ((GList *) history);
     g_autofree const gchar **displayed_history = g_new (const gchar *, length + 1);
 
-    for (guint64 i = 0; i < length; ++i, history = g_slist_next (history))
+    for (guint64 i = 0; i < length; ++i, history = g_list_next (history))
         displayed_history[i] = g_paste_item_get_value (history->data);
     displayed_history[length] = NULL;
 
@@ -572,9 +566,9 @@ g_paste_daemon_list_histories (GError **error)
 }
 
 static void
-g_paste_daemon_private_merge (GPasteDaemonPrivate *priv,
-                              GVariant            *parameters,
-                              GPasteDBusError    **err)
+g_paste_daemon_private_merge (const GPasteDaemonPrivate *priv,
+                              GVariant                  *parameters,
+                              GPasteDBusError          **err)
 {
     GVariantIter parameters_iter;
 
@@ -623,7 +617,7 @@ g_paste_daemon_track (GPasteDaemon *self,
     g_autoptr (GVariant) variant = g_variant_iter_next_value (&parameters_iter);
     gboolean tracking_state = g_variant_get_boolean (variant);
 
-    GPasteDaemonPrivate *priv = g_paste_daemon_get_instance_private (self);
+    const GPasteDaemonPrivate *priv = _g_paste_daemon_get_instance_private (self);
 
     g_paste_settings_set_track_changes (priv->settings, tracking_state);
     g_paste_daemon_tracking (self, tracking_state, NULL);
@@ -633,7 +627,7 @@ static void
 g_paste_daemon_on_extension_state_changed (GPasteDaemon *self,
                                            GVariant     *parameters)
 {
-    GPasteDaemonPrivate *priv = g_paste_daemon_get_instance_private (self);
+    const GPasteDaemonPrivate *priv = _g_paste_daemon_get_instance_private (self);
 
     if (g_paste_settings_get_track_extension_state (priv->settings))
         g_paste_daemon_track (self, parameters);
@@ -649,9 +643,9 @@ g_paste_daemon_reexecute (GPasteDaemon *self)
 }
 
 static void
-g_paste_daemon_private_rename_password (GPasteDaemonPrivate *priv,
-                                        GVariant            *parameters,
-                                        GPasteDBusError    **err)
+g_paste_daemon_private_rename_password (const GPasteDaemonPrivate *priv,
+                                        GVariant                  *parameters,
+                                        GPasteDBusError          **err)
 {
     g_autofree gchar *old_name = NULL;
     g_autofree gchar *new_name = NULL;
@@ -664,7 +658,7 @@ g_paste_daemon_private_rename_password (GPasteDaemonPrivate *priv,
 }
 
 static GVariant *
-g_paste_daemon_private_search (GPasteDaemonPrivate *priv,
+g_paste_daemon_private_search (const GPasteDaemonPrivate *priv,
                                GVariant                  *parameters,
                                GPasteDBusError          **err)
 {
@@ -679,17 +673,17 @@ g_paste_daemon_private_search (GPasteDaemonPrivate *priv,
 }
 
 static void
-g_paste_daemon_private_select (GPasteDaemonPrivate *priv,
-                               GVariant            *parameters)
+g_paste_daemon_private_select (const GPasteDaemonPrivate *priv,
+                               GVariant                  *parameters)
 {
     g_paste_history_select (priv->history,
                             g_paste_daemon_get_dbus_uint64_parameter (parameters));
 }
 
 static void
-g_paste_daemon_private_replace (GPasteDaemonPrivate *priv,
-                                GVariant            *parameters,
-                                GPasteDBusError    **err)
+g_paste_daemon_private_replace (const GPasteDaemonPrivate *priv,
+                                GVariant                  *parameters,
+                                GPasteDBusError          **err)
 {
     GPasteHistory *history = priv->history;
     GVariantIter parameters_iter;
@@ -705,7 +699,7 @@ g_paste_daemon_private_replace (GPasteDaemonPrivate *priv,
     const GPasteItem *item = g_paste_history_get (history, index);
 
     G_PASTE_DBUS_ASSERT (item, "received no item for this index");
-    G_PASTE_DBUS_ASSERT (G_PASTE_IS_TEXT_ITEM (item) && !g_strcmp0 (g_paste_item_get_kind (item), "Text"), "attempted to replace an item other than GPasteTextItem");
+    G_PASTE_DBUS_ASSERT (_G_PASTE_IS_TEXT_ITEM (item) && g_paste_str_equal (g_paste_item_get_kind (item), "Text"), "attempted to replace an item other than GPasteTextItem");
 
     g_autoptr (GVariant) variant2 = g_variant_iter_next_value (&parameters_iter);
     g_autofree gchar *contents = g_variant_dup_string (variant2, &length);
@@ -716,9 +710,9 @@ g_paste_daemon_private_replace (GPasteDaemonPrivate *priv,
 }
 
 static void
-g_paste_daemon_private_set_password (GPasteDaemonPrivate *priv,
-                                     GVariant            *parameters,
-                                     GPasteDBusError    **err)
+g_paste_daemon_private_set_password (const GPasteDaemonPrivate *priv,
+                                     GVariant                  *parameters,
+                                     GPasteDBusError          **err)
 {
     GPasteHistory *history = priv->history;
     GVariantIter parameters_iter;
@@ -734,7 +728,7 @@ g_paste_daemon_private_set_password (GPasteDaemonPrivate *priv,
     const GPasteItem *item = g_paste_history_get (history, index);
 
     G_PASTE_DBUS_ASSERT (item, "received no item for this index");
-    G_PASTE_DBUS_ASSERT (G_PASTE_IS_TEXT_ITEM (item) && !g_strcmp0 (g_paste_item_get_kind (item), "Text"), "attempted to replace an item other than GPasteTextItem");
+    G_PASTE_DBUS_ASSERT (_G_PASTE_IS_TEXT_ITEM (item) && g_paste_str_equal (g_paste_item_get_kind (item), "Text"), "attempted to replace an item other than GPasteTextItem");
 
     g_autoptr (GVariant) variant2 = g_variant_iter_next_value (&parameters_iter);
     g_autofree gchar *name = g_variant_dup_string (variant2, &length);
@@ -745,9 +739,9 @@ g_paste_daemon_private_set_password (GPasteDaemonPrivate *priv,
 }
 
 static void
-g_paste_daemon_private_switch_history (GPasteDaemonPrivate *priv,
-                                       GVariant            *parameters,
-                                       GPasteDBusError    **err)
+g_paste_daemon_private_switch_history (const GPasteDaemonPrivate *priv,
+                                       GVariant                  *parameters,
+                                       GPasteDBusError          **err)
 {
     g_autofree gchar *name = g_paste_daemon_get_dbus_string_parameter (parameters, NULL);
 
@@ -782,14 +776,12 @@ g_paste_daemon_private_upload_finish (GObject      *source_object,
  * @index: the index of the item to upload
  *
  * Upload an item to a pastebin service
- *
- * Returns:
  */
 G_PASTE_VISIBLE void
 g_paste_daemon_upload (GPasteDaemon *self,
                        guint64       index)
 {
-    g_return_if_fail (G_PASTE_IS_DAEMON (self));
+    g_return_if_fail (_G_PASTE_IS_DAEMON (self));
 
     GPasteDaemonPrivate *priv = g_paste_daemon_get_instance_private (self);
     GSubprocess *upload = g_subprocess_new (G_SUBPROCESS_FLAGS_STDIN_PIPE|G_SUBPROCESS_FLAGS_STDOUT_PIPE, NULL, "wgetpaste", NULL);
@@ -812,7 +804,7 @@ _g_paste_daemon_upload (GPasteDaemon *self,
 static void
 g_paste_daemon_activate_default_keybindings (GPasteDaemon *self)
 {
-    GPasteDaemonPrivate *priv = g_paste_daemon_get_instance_private (self);
+    const GPasteDaemonPrivate *priv = _g_paste_daemon_get_instance_private (self);
     GPasteKeybinder *keybinder = priv->keybinder;
     GPasteHistory *history = priv->history;
     GPasteClipboardsManager *clipboards_manager = priv->clipboards_manager;
@@ -843,70 +835,70 @@ g_paste_daemon_dbus_method_call (GDBusConnection       *connection     G_GNUC_UN
                                  gpointer               user_data)
 {
     GPasteDaemon *self = user_data;
-    GPasteDaemonPrivate *priv = g_paste_daemon_get_instance_private (self);
+    const GPasteDaemonPrivate *priv = _g_paste_daemon_get_instance_private (self);
     GVariant *answer = NULL;
     GError *error = NULL;
     g_autofree GPasteDBusError *err = NULL;
 
-    if (!g_strcmp0 (method_name, G_PASTE_DAEMON_ABOUT))
+    if (g_paste_str_equal (method_name, G_PASTE_DAEMON_ABOUT))
         g_paste_util_activate_ui ("about", NULL);
-    else if (!g_strcmp0 (method_name, G_PASTE_DAEMON_ADD))
+    else if (g_paste_str_equal (method_name, G_PASTE_DAEMON_ADD))
         g_paste_daemon_private_add (priv, parameters, &err);
-    else if (!g_strcmp0 (method_name, G_PASTE_DAEMON_ADD_FILE))
+    else if (g_paste_str_equal (method_name, G_PASTE_DAEMON_ADD_FILE))
         g_paste_daemon_private_add_file (priv, parameters, &error, &err);
-    else if (!g_strcmp0 (method_name, G_PASTE_DAEMON_ADD_PASSWORD))
+    else if (g_paste_str_equal (method_name, G_PASTE_DAEMON_ADD_PASSWORD))
         g_paste_daemon_private_add_password (priv, parameters, &err);
-    else if (!g_strcmp0 (method_name, G_PASTE_DAEMON_BACKUP_HISTORY))
+    else if (g_paste_str_equal (method_name, G_PASTE_DAEMON_BACKUP_HISTORY))
         g_paste_daemon_private_backup_history (priv, parameters, &err);
-    else if (!g_strcmp0 (method_name, G_PASTE_DAEMON_DELETE))
+    else if (g_paste_str_equal (method_name, G_PASTE_DAEMON_DELETE))
         g_paste_daemon_private_delete (priv, parameters);
-    else if (!g_strcmp0 (method_name, G_PASTE_DAEMON_DELETE_HISTORY))
+    else if (g_paste_str_equal (method_name, G_PASTE_DAEMON_DELETE_HISTORY))
         g_paste_daemon_private_delete_history (priv, parameters, &err);
-    else if (!g_strcmp0 (method_name, G_PASTE_DAEMON_DELETE_PASSWORD))
+    else if (g_paste_str_equal (method_name, G_PASTE_DAEMON_DELETE_PASSWORD))
         g_paste_daemon_private_delete_password (priv, parameters, &err);
-    else if (!g_strcmp0 (method_name, G_PASTE_DAEMON_EMPTY_HISTORY))
+    else if (g_paste_str_equal (method_name, G_PASTE_DAEMON_EMPTY_HISTORY))
         g_paste_daemon_private_empty_history (priv, parameters);
-    else if (!g_strcmp0 (method_name, G_PASTE_DAEMON_GET_ELEMENT))
+    else if (g_paste_str_equal (method_name, G_PASTE_DAEMON_GET_ELEMENT))
         answer = g_paste_daemon_private_get_element (priv, parameters, &err);
-    else if (!g_strcmp0 (method_name, G_PASTE_DAEMON_GET_ELEMENT_KIND))
+    else if (g_paste_str_equal (method_name, G_PASTE_DAEMON_GET_ELEMENT_KIND))
         answer = g_paste_daemon_private_get_element_kind (priv, parameters, &err);
-    else if (!g_strcmp0 (method_name, G_PASTE_DAEMON_GET_ELEMENTS))
+    else if (g_paste_str_equal (method_name, G_PASTE_DAEMON_GET_ELEMENTS))
         answer = g_paste_daemon_private_get_elements (priv, parameters, &err);
-    else if (!g_strcmp0 (method_name, G_PASTE_DAEMON_GET_HISTORY))
+    else if (g_paste_str_equal (method_name, G_PASTE_DAEMON_GET_HISTORY))
         answer = g_paste_daemon_private_get_history (priv);
-    else if (!g_strcmp0 (method_name, G_PASTE_DAEMON_GET_HISTORY_NAME))
+    else if (g_paste_str_equal (method_name, G_PASTE_DAEMON_GET_HISTORY_NAME))
         answer = g_paste_daemon_private_get_history_name (priv);
-    else if (!g_strcmp0 (method_name, G_PASTE_DAEMON_GET_HISTORY_SIZE))
+    else if (g_paste_str_equal (method_name, G_PASTE_DAEMON_GET_HISTORY_SIZE))
         answer = g_paste_daemon_private_get_history_size (priv, parameters);
-    else if (!g_strcmp0 (method_name, G_PASTE_DAEMON_GET_RAW_ELEMENT))
+    else if (g_paste_str_equal (method_name, G_PASTE_DAEMON_GET_RAW_ELEMENT))
         answer = g_paste_daemon_private_get_raw_element (priv, parameters, &err);
-    else if (!g_strcmp0 (method_name, G_PASTE_DAEMON_GET_RAW_HISTORY))
+    else if (g_paste_str_equal (method_name, G_PASTE_DAEMON_GET_RAW_HISTORY))
         answer = g_paste_daemon_private_get_raw_history (priv);
-    else if (!g_strcmp0 (method_name, G_PASTE_DAEMON_LIST_HISTORIES))
+    else if (g_paste_str_equal (method_name, G_PASTE_DAEMON_LIST_HISTORIES))
         answer = g_paste_daemon_list_histories (&error);
-    else if (!g_strcmp0 (method_name, G_PASTE_DAEMON_MERGE))
+    else if (g_paste_str_equal (method_name, G_PASTE_DAEMON_MERGE))
         g_paste_daemon_private_merge (priv, parameters, &err);
-    else if (!g_strcmp0 (method_name, G_PASTE_DAEMON_ON_EXTENSION_STATE_CHANGED))
+    else if (g_paste_str_equal (method_name, G_PASTE_DAEMON_ON_EXTENSION_STATE_CHANGED))
         g_paste_daemon_on_extension_state_changed (self, parameters);
-    else if (!g_strcmp0 (method_name, G_PASTE_DAEMON_REEXECUTE))
+    else if (g_paste_str_equal (method_name, G_PASTE_DAEMON_REEXECUTE))
         g_paste_daemon_reexecute (self);
-    else if (!g_strcmp0 (method_name, G_PASTE_DAEMON_RENAME_PASSWORD))
+    else if (g_paste_str_equal (method_name, G_PASTE_DAEMON_RENAME_PASSWORD))
         g_paste_daemon_private_rename_password (priv, parameters, &err);
-    else if (!g_strcmp0 (method_name, G_PASTE_DAEMON_REPLACE))
+    else if (g_paste_str_equal (method_name, G_PASTE_DAEMON_REPLACE))
         g_paste_daemon_private_replace (priv, parameters, &err);
-    else if (!g_strcmp0 (method_name, G_PASTE_DAEMON_SEARCH))
+    else if (g_paste_str_equal (method_name, G_PASTE_DAEMON_SEARCH))
         answer = g_paste_daemon_private_search (priv, parameters, &err);
-    else if (!g_strcmp0 (method_name, G_PASTE_DAEMON_SELECT))
+    else if (g_paste_str_equal (method_name, G_PASTE_DAEMON_SELECT))
         g_paste_daemon_private_select (priv, parameters);
-    else if (!g_strcmp0 (method_name, G_PASTE_DAEMON_SET_PASSWORD))
+    else if (g_paste_str_equal (method_name, G_PASTE_DAEMON_SET_PASSWORD))
         g_paste_daemon_private_set_password (priv, parameters, &err);
-    else if (!g_strcmp0 (method_name, G_PASTE_DAEMON_SHOW_HISTORY))
+    else if (g_paste_str_equal (method_name, G_PASTE_DAEMON_SHOW_HISTORY))
         g_paste_daemon_show_history (self, &error);
-    else if (!g_strcmp0 (method_name, G_PASTE_DAEMON_SWITCH_HISTORY))
+    else if (g_paste_str_equal (method_name, G_PASTE_DAEMON_SWITCH_HISTORY))
         g_paste_daemon_private_switch_history (priv, parameters, &err);
-    else if (!g_strcmp0 (method_name, G_PASTE_DAEMON_TRACK))
+    else if (g_paste_str_equal (method_name, G_PASTE_DAEMON_TRACK))
         g_paste_daemon_track (self, parameters);
-    else if (!g_strcmp0 (method_name, G_PASTE_DAEMON_UPLOAD))
+    else if (g_paste_str_equal (method_name, G_PASTE_DAEMON_UPLOAD))
         _g_paste_daemon_upload (self, parameters);
 
     if (error)
@@ -926,11 +918,11 @@ g_paste_daemon_dbus_get_property (GDBusConnection *connection G_GNUC_UNUSED,
                                   GError         **error G_GNUC_UNUSED,
                                   gpointer         user_data)
 {
-    GPasteDaemonPrivate *priv = g_paste_daemon_get_instance_private (G_PASTE_DAEMON (user_data));
+    const GPasteDaemonPrivate *priv = _g_paste_daemon_get_instance_private (G_PASTE_DAEMON (user_data));
 
-    if (!g_strcmp0 (property_name, G_PASTE_DAEMON_PROP_ACTIVE))
+    if (g_paste_str_equal (property_name, G_PASTE_DAEMON_PROP_ACTIVE))
         return g_variant_new_boolean (g_paste_settings_get_track_changes (priv->settings));
-    else if (!g_strcmp0 (property_name, G_PASTE_DAEMON_PROP_VERSION))
+    else if (g_paste_str_equal (property_name, G_PASTE_DAEMON_PROP_VERSION))
         return g_variant_new_string (VERSION);
 
     return NULL;
@@ -941,14 +933,14 @@ g_paste_daemon_unregister_object (gpointer user_data)
 {
     g_autoptr (GPasteDaemon) self = G_PASTE_DAEMON (user_data);
     GPasteDaemonPrivate *priv = g_paste_daemon_get_instance_private (self);
-    gulong *c_signals = priv->c_signals;
+    guint64 *c_signals = priv->c_signals;
 
     g_signal_handler_disconnect (priv->settings, c_signals[C_TRACK]);
     g_signal_handler_disconnect (priv->history,  c_signals[C_UPDATE]);
     g_signal_handler_disconnect (priv->history,  c_signals[C_SWITCH]);
 
     if (priv->screensaver)
-        g_signal_handler_disconnect (priv->screensaver,  c_signals[C_ACTIVE_CHANGED]);
+        g_signal_handler_disconnect (priv->screensaver, c_signals[C_ACTIVE_CHANGED]);
 
     priv->registered = FALSE;
 }
@@ -983,19 +975,15 @@ g_paste_daemon_on_screensaver_active_changed (GPasteDaemonPrivate *priv,
     /* We always do the activate action, so that the deactivate one works anyways */
     {
         g_autoptr (GPasteItem) item = g_paste_text_item_new ("");
-        /* will always return TRUE */
         g_paste_clipboards_manager_select (priv->clipboards_manager, item);
     }
 
     if (!active)
     {
-        const GPasteItem *item = g_paste_history_get (priv->history, 0);
+        g_autoptr (GPasteItem) item = g_paste_history_dup (priv->history, 0);
 
         if (item)
-        {
-            if (!g_paste_clipboards_manager_select (priv->clipboards_manager, item))
-                g_paste_history_remove (priv->history, 0);
-        }
+            g_paste_clipboards_manager_select (priv->clipboards_manager, item);
     }
 }
 
@@ -1012,7 +1000,7 @@ _g_paste_daemon_changed (gpointer data)
 static void
 g_paste_daemon_dispose (GObject *object)
 {
-    GPasteDaemonPrivate *priv = g_paste_daemon_get_instance_private (G_PASTE_DAEMON (object));
+    const GPasteDaemonPrivate *priv = _g_paste_daemon_get_instance_private (G_PASTE_DAEMON (object));
 
     if (priv->settings)
     {
@@ -1050,7 +1038,7 @@ g_paste_daemon_register_on_connection (GPasteBusObject *self,
     if (!priv->id_on_bus)
         return FALSE;
 
-    gulong *c_signals = priv->c_signals;
+    guint64 *c_signals = priv->c_signals;
 
     c_signals[C_TRACK] = g_signal_connect_swapped (priv->settings,
                                                    "track",
