@@ -1,30 +1,26 @@
 /*
- *      This file is part of GPaste.
+ * This file is part of GPaste.
  *
- *      Copyright 2011-2015 Marc-Antoine Perennou <Marc-Antoine@Perennou.com>
- *
- *      GPaste is free software: you can redistribute it and/or modify
- *      it under the terms of the GNU General Public License as published by
- *      the Free Software Foundation, either version 3 of the License, or
- *      (at your option) any later version.
- *
- *      GPaste is distributed in the hope that it will be useful,
- *      but WITHOUT ANY WARRANTY; without even the implied warranty of
- *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *      GNU General Public License for more details.
- *
- *      You should have received a copy of the GNU General Public License
- *      along with GPaste.  If not, see <http://www.gnu.org/licenses/>.
+ * Copyright (c) 2010-2016, Marc-Antoine Perennou <Marc-Antoine@Perennou.com>
  */
 
 #include <gpaste-gsettings-keys.h>
 #include <gpaste-settings.h>
+#include <gpaste-util.h>
 
 struct _GPasteSettings
 {
     GObject parent_instance;
 };
 
+enum
+{
+    C_CHANGED,
+    C_SHELL_CHANGED,
+
+    C_LAST_SIGNAL
+};
+
 typedef struct
 {
     GSettings *settings;
@@ -55,11 +51,10 @@ typedef struct
 
     gboolean   extension_enabled;
 
-    gulong     changed_signal;
-    gulong     shell_changed_signal;
+    guint64    c_signals[C_LAST_SIGNAL];
 } GPasteSettingsPrivate;
 
-G_DEFINE_TYPE_WITH_PRIVATE (GPasteSettings, g_paste_settings, G_TYPE_OBJECT)
+G_PASTE_DEFINE_TYPE_WITH_PRIVATE (Settings, settings, G_TYPE_OBJECT)
 
 enum
 {
@@ -76,15 +71,15 @@ static guint64 signals[LAST_SIGNAL] = { 0 };
     G_PASTE_VISIBLE type                                                                               \
     g_paste_settings_get_##name (const GPasteSettings *self)                                           \
     {                                                                                                  \
-        g_return_val_if_fail (G_PASTE_IS_SETTINGS ((gpointer) self), fail);                                       \
-        GPasteSettingsPrivate *priv = g_paste_settings_get_instance_private (self); \
+        g_return_val_if_fail (_G_PASTE_IS_SETTINGS ((gpointer) self), fail);                           \
+        const GPasteSettingsPrivate *priv = _g_paste_settings_get_instance_private (self);             \
         return priv->name;                                                                             \
     }                                                                                                  \
     G_PASTE_VISIBLE void                                                                               \
     g_paste_settings_reset_##name (GPasteSettings *self)                                               \
     {                                                                                                  \
-        g_return_if_fail (G_PASTE_IS_SETTINGS (self));                                                 \
-        GPasteSettingsPrivate *priv = g_paste_settings_get_instance_private (self); \
+        g_return_if_fail (_G_PASTE_IS_SETTINGS (self));                                                \
+        const GPasteSettingsPrivate *priv = _g_paste_settings_get_instance_private (self);             \
         g_settings_reset (priv->settings, G_PASTE_##key##_SETTING);                                    \
     }                                                                                                  \
     static void                                                                                        \
@@ -96,7 +91,7 @@ static guint64 signals[LAST_SIGNAL] = { 0 };
     g_paste_settings_set_##name (GPasteSettings *self,                                                 \
                                  type            value)                                                \
     {                                                                                                  \
-        g_return_if_fail (G_PASTE_IS_SETTINGS (self));                                                 \
+        g_return_if_fail (_G_PASTE_IS_SETTINGS (self));                                                \
         guards                                                                                         \
         GPasteSettingsPrivate *priv = g_paste_settings_get_instance_private (self);                    \
         clear_func                                                                                     \
@@ -160,8 +155,6 @@ g_settings_set_gpaste_uint64 (GSettings   *settings,
  * @self: a #GPasteSettings instance
  *
  * Reset the "element-size" setting
- *
- * Returns:
  */
 /**
  * g_paste_settings_set_element_size:
@@ -169,8 +162,6 @@ g_settings_set_gpaste_uint64 (GSettings   *settings,
  * @value: the maximum displayed size of an item
  *
  * Change the "element-size" setting
- *
- * Returns:
  */
 UNSIGNED_SETTING (element_size, ELEMENT_SIZE)
 
@@ -187,8 +178,6 @@ UNSIGNED_SETTING (element_size, ELEMENT_SIZE)
  * @self: a #GPasteSettings instance
  *
  * Reset the "growing-lines" setting
- *
- * Returns:
  */
 /**
  * g_paste_settings_set_growing_lines:
@@ -196,8 +185,6 @@ UNSIGNED_SETTING (element_size, ELEMENT_SIZE)
  * @value: whether to detect or not growing lines
  *
  * Change the "growing-lines" setting
- *
- * Returns:
  */
 BOOLEAN_SETTING (growing_lines, GROWING_LINES)
 
@@ -214,8 +201,6 @@ BOOLEAN_SETTING (growing_lines, GROWING_LINES)
  * @self: a #GPasteSettings instance
  *
  * Reset the "history-name" setting
- *
- * Returns:
  */
 /**
  * g_paste_settings_set_history_name:
@@ -223,8 +208,6 @@ BOOLEAN_SETTING (growing_lines, GROWING_LINES)
  * @value: the new history name
  *
  * Change the "history-name" setting
- *
- * Returns:
  */
 STRING_SETTING (history_name, HISTORY_NAME)
 
@@ -241,8 +224,6 @@ STRING_SETTING (history_name, HISTORY_NAME)
  * @self: a #GPasteSettings instance
  *
  * Reset the "images-support" setting
- *
- * Returns:
  */
 /**
  * g_paste_settings_set_images_support:
@@ -250,8 +231,6 @@ STRING_SETTING (history_name, HISTORY_NAME)
  * @value: the new history name
  *
  * Change the "images-support" setting
- *
- * Returns:
  */
 BOOLEAN_SETTING (images_support, IMAGES_SUPPORT)
 
@@ -268,8 +247,6 @@ BOOLEAN_SETTING (images_support, IMAGES_SUPPORT)
  * @self: a #GPasteSettings instance
  *
  * Reset the "launch-ui" setting
- *
- * Returns:
  */
 /**
  * g_paste_settings_set_launch_ui:
@@ -277,8 +254,6 @@ BOOLEAN_SETTING (images_support, IMAGES_SUPPORT)
  * @value: the new keyboard shortcut
  *
  * Change the "launch-ui" setting
- *
- * Returns:
  */
 STRING_SETTING (launch_ui, LAUNCH_UI)
 
@@ -295,8 +270,6 @@ STRING_SETTING (launch_ui, LAUNCH_UI)
  * @self: a #GPasteSettings instance
  *
  * Reset the "make-password" setting
- *
- * Returns:
  */
 /**
  * g_paste_settings_set_make_password:
@@ -304,8 +277,6 @@ STRING_SETTING (launch_ui, LAUNCH_UI)
  * @value: the new keyboard shortcut
  *
  * Change the "make-password" setting
- *
- * Returns:
  */
 STRING_SETTING (make_password, MAKE_PASSWORD)
 
@@ -322,8 +293,6 @@ STRING_SETTING (make_password, MAKE_PASSWORD)
  * @self: a #GPasteSettings instance
  *
  * Reset the "max-displayed-history-size" setting
- *
- * Returns:
  */
 /**
  * g_paste_settings_set_max_displayed_history_size:
@@ -331,8 +300,6 @@ STRING_SETTING (make_password, MAKE_PASSWORD)
  * @value: the maximum number of items to display
  *
  * Change the "max-displayed-history-size" setting
- *
- * Returns:
  */
 UNSIGNED_SETTING (max_displayed_history_size, MAX_DISPLAYED_HISTORY_SIZE)
 
@@ -349,8 +316,6 @@ UNSIGNED_SETTING (max_displayed_history_size, MAX_DISPLAYED_HISTORY_SIZE)
  * @self: a #GPasteSettings instance
  *
  * Reset the "max-history-size" setting
- *
- * Returns:
  */
 /**
  * g_paste_settings_set_max_history_size:
@@ -358,8 +323,6 @@ UNSIGNED_SETTING (max_displayed_history_size, MAX_DISPLAYED_HISTORY_SIZE)
  * @value: the maximum number of items the history can contain
  *
  * Change the "max-history-size" setting
- *
- * Returns:
  */
 UNSIGNED_SETTING (max_history_size, MAX_HISTORY_SIZE)
 
@@ -376,8 +339,6 @@ UNSIGNED_SETTING (max_history_size, MAX_HISTORY_SIZE)
  * @self: a #GPasteSettings instance
  *
  * Reset the "max-memory-usage" setting
- *
- * Returns:
  */
 /**
  * g_paste_settings_set_max_memory_usage:
@@ -385,8 +346,6 @@ UNSIGNED_SETTING (max_history_size, MAX_HISTORY_SIZE)
  * @value: the maximum amout of memory we can use
  *
  * Change the "max-memory-usage" setting
- *
- * Returns:
  */
 UNSIGNED_SETTING (max_memory_usage, MAX_MEMORY_USAGE)
 
@@ -403,8 +362,6 @@ UNSIGNED_SETTING (max_memory_usage, MAX_MEMORY_USAGE)
  * @self: a #GPasteSettings instance
  *
  * Reset the "max-text-item-size" setting
- *
- * Returns:
  */
 /**
  * g_paste_settings_set_max_text_item_size:
@@ -412,8 +369,6 @@ UNSIGNED_SETTING (max_memory_usage, MAX_MEMORY_USAGE)
  * @value: the maximum size for a textual item to be handled
  *
  * Change the "max-text-item-size" setting
- *
- * Returns:
  */
 UNSIGNED_SETTING (max_text_item_size, MAX_TEXT_ITEM_SIZE)
 
@@ -430,8 +385,6 @@ UNSIGNED_SETTING (max_text_item_size, MAX_TEXT_ITEM_SIZE)
  * @self: a #GPasteSettings instance
  *
  * Reset the "min-text-item-size" setting
- *
- * Returns:
  */
 /**
  * g_paste_settings_set_min_text_item_size:
@@ -439,8 +392,6 @@ UNSIGNED_SETTING (max_text_item_size, MAX_TEXT_ITEM_SIZE)
  * @value: the minimum size for a textual item to be handled
  *
  * Change the "min-text-item-size" setting
- *
- * Returns:
  */
 UNSIGNED_SETTING (min_text_item_size, MIN_TEXT_ITEM_SIZE)
 
@@ -457,8 +408,6 @@ UNSIGNED_SETTING (min_text_item_size, MIN_TEXT_ITEM_SIZE)
  * @self: a #GPasteSettings instance
  *
  * Reset the "pop" setting
- *
- * Returns:
  */
 /**
  * g_paste_settings_set_pop:
@@ -466,8 +415,6 @@ UNSIGNED_SETTING (min_text_item_size, MIN_TEXT_ITEM_SIZE)
  * @value: the new keyboard shortcut
  *
  * Change the "pop" setting
- *
- * Returns:
  */
 STRING_SETTING (pop, POP)
 
@@ -484,8 +431,6 @@ STRING_SETTING (pop, POP)
  * @self: a #GPasteSettings instance
  *
  * Reset the "primary-to-history" setting
- *
- * Returns:
  */
 /**
  * g_paste_settings_set_primary_to_history:
@@ -493,8 +438,6 @@ STRING_SETTING (pop, POP)
  * @value: whether to track or not the primary selection changes as clipboard ones
  *
  * Change the "primary-to-history" setting
- *
- * Returns:
  */
 BOOLEAN_SETTING (primary_to_history, PRIMARY_TO_HISTORY)
 
@@ -511,8 +454,6 @@ BOOLEAN_SETTING (primary_to_history, PRIMARY_TO_HISTORY)
  * @self: a #GPasteSettings instance
  *
  * Reset the "save-history" setting
- *
- * Returns:
  */
 /**
  * g_paste_settings_set_save_history:
@@ -520,8 +461,6 @@ BOOLEAN_SETTING (primary_to_history, PRIMARY_TO_HISTORY)
  * @value: whether to save or not the history
  *
  * Change the "save-history" setting
- *
- * Returns:
  */
 BOOLEAN_SETTING (save_history, SAVE_HISTORY)
 
@@ -538,8 +477,6 @@ BOOLEAN_SETTING (save_history, SAVE_HISTORY)
  * @self: a #GPasteSettings instance
  *
  * Reset the "show-history" setting
- *
- * Returns:
  */
 /**
  * g_paste_settings_set_show_history:
@@ -547,8 +484,6 @@ BOOLEAN_SETTING (save_history, SAVE_HISTORY)
  * @value: the new keyboard shortcut
  *
  * Change the "show-history" setting
- *
- * Returns:
  */
 STRING_SETTING (show_history, SHOW_HISTORY)
 
@@ -565,8 +500,6 @@ STRING_SETTING (show_history, SHOW_HISTORY)
  * @self: a #GPasteSettings instance
  *
  * Reset the "sync-clipboard-to-primary" setting
- *
- * Returns:
  */
 /**
  * g_paste_settings_set_sync_clipboard_to_primary:
@@ -574,8 +507,6 @@ STRING_SETTING (show_history, SHOW_HISTORY)
  * @value: the new keyboard shortcut
  *
  * Change the "sync-clipboard-to-primary" setting
- *
- * Returns:
  */
 STRING_SETTING (sync_clipboard_to_primary, SYNC_CLIPBOARD_TO_PRIMARY)
 
@@ -592,8 +523,6 @@ STRING_SETTING (sync_clipboard_to_primary, SYNC_CLIPBOARD_TO_PRIMARY)
  * @self: a #GPasteSettings instance
  *
  * Reset the "sync-primary-to-clipboard" setting
- *
- * Returns:
  */
 /**
  * g_paste_settings_set_sync_primary_to_clipboard:
@@ -601,8 +530,6 @@ STRING_SETTING (sync_clipboard_to_primary, SYNC_CLIPBOARD_TO_PRIMARY)
  * @value: the new keyboard shortcut
  *
  * Change the "sync-primary-to-clipboard" setting
- *
- * Returns:
  */
 STRING_SETTING (sync_primary_to_clipboard, SYNC_PRIMARY_TO_CLIPBOARD)
 
@@ -619,8 +546,6 @@ STRING_SETTING (sync_primary_to_clipboard, SYNC_PRIMARY_TO_CLIPBOARD)
  * @self: a #GPasteSettings instance
  *
  * Reset the "synchronize-clipboards" setting
- *
- * Returns:
  */
 /**
  * g_paste_settings_set_synchronize_clipboards:
@@ -628,8 +553,6 @@ STRING_SETTING (sync_primary_to_clipboard, SYNC_PRIMARY_TO_CLIPBOARD)
  * @value: whether to synchronize the clipboard and the primary selection or not
  *
  * Change the "synchronize-clipboards" setting
- *
- * Returns:
  */
 BOOLEAN_SETTING (synchronize_clipboards, SYNCHRONIZE_CLIPBOARDS)
 
@@ -646,8 +569,6 @@ BOOLEAN_SETTING (synchronize_clipboards, SYNCHRONIZE_CLIPBOARDS)
  * @self: a #GPasteSettings instance
  *
  * Reset the "track-changes" setting
- *
- * Returns:
  */
 /**
  * g_paste_settings_set_track_changes:
@@ -655,8 +576,6 @@ BOOLEAN_SETTING (synchronize_clipboards, SYNCHRONIZE_CLIPBOARDS)
  * @value: whether to track or not the clipboard changes
  *
  * Change the "track-changes" setting
- *
- * Returns:
  */
 BOOLEAN_SETTING (track_changes, TRACK_CHANGES)
 
@@ -673,8 +592,6 @@ BOOLEAN_SETTING (track_changes, TRACK_CHANGES)
  * @self: a #GPasteSettings instance
  *
  * Reset the "track-extension-state" setting
- *
- * Returns:
  */
 /**
  * g_paste_settings_set_track_extension_state:
@@ -682,8 +599,6 @@ BOOLEAN_SETTING (track_changes, TRACK_CHANGES)
  * @value: whether to stop tracking or not the clipboard changes when an applet exits
  *
  * Change the "track-extension-state" setting
- *
- * Returns:
  */
 BOOLEAN_SETTING (track_extension_state, TRACK_EXTENSION_STATE)
 
@@ -700,8 +615,6 @@ BOOLEAN_SETTING (track_extension_state, TRACK_EXTENSION_STATE)
  * @self: a #GPasteSettings instance
  *
  * Reset the "trim-items" setting
- *
- * Returns:
  */
 /**
  * g_paste_settings_set_trim_items:
@@ -709,8 +622,6 @@ BOOLEAN_SETTING (track_extension_state, TRACK_EXTENSION_STATE)
  * @value: whether to trim or not textual items
  *
  * Change the "trim-items" setting
- *
- * Returns:
  */
 BOOLEAN_SETTING (trim_items, TRIM_ITEMS)
 
@@ -727,8 +638,6 @@ BOOLEAN_SETTING (trim_items, TRIM_ITEMS)
  * @self: a #GPasteSettings instance
  *
  * Reset the "upload" setting
- *
- * Returns:
  */
 /**
  * g_paste_settings_set_upload:
@@ -736,13 +645,9 @@ BOOLEAN_SETTING (trim_items, TRIM_ITEMS)
  * @value: the new keyboard shortcut
  *
  * Change the "upload" setting
- *
- * Returns:
  */
 STRING_SETTING (upload, UPLOAD)
 
-#ifdef ENABLE_EXTENSION
-#define EXTENSION_NAME "GPaste@gnome-shell-extensions.gnome.org"
 /**
  * g_paste_settings_get_extension_enabled:
  * @self: a #GPasteSettings instance
@@ -754,8 +659,8 @@ STRING_SETTING (upload, UPLOAD)
 G_PASTE_VISIBLE gboolean
 g_paste_settings_get_extension_enabled (const GPasteSettings *self)
 {
-    g_return_val_if_fail (G_PASTE_IS_SETTINGS ((gpointer) self), FALSE);
-    GPasteSettingsPrivate *priv = g_paste_settings_get_instance_private (self);
+    g_return_val_if_fail (_G_PASTE_IS_SETTINGS ((gpointer) self), FALSE);
+    const GPasteSettingsPrivate *priv = _g_paste_settings_get_instance_private (self);
     return priv->extension_enabled;
 }
 
@@ -771,7 +676,7 @@ g_paste_settings_private_set_extension_enabled_from_dconf (GPasteSettingsPrivate
     g_auto (GStrv) extensions = g_paste_settings_private_get_enabled_extensions (priv);
     for (GStrv e = extensions; *e; ++e)
     {
-        if (!g_strcmp0 (*e, EXTENSION_NAME))
+        if (g_paste_str_equal (*e, G_PASTE_EXTENSION_NAME))
         {
             priv->extension_enabled = TRUE;
             return;
@@ -786,16 +691,16 @@ g_paste_settings_private_set_extension_enabled_from_dconf (GPasteSettingsPrivate
  * @value: whether to enable or not the gnome-shell extension
  *
  * Change the "extension-enabled" special setting
- *
- * Returns:
  */
 G_PASTE_VISIBLE void
 g_paste_settings_set_extension_enabled (GPasteSettings *self,
                                         gboolean        value)
 {
-    g_return_if_fail (G_PASTE_IS_SETTINGS (self));
+    g_return_if_fail (_G_PASTE_IS_SETTINGS (self));
+
     GPasteSettingsPrivate *priv = g_paste_settings_get_instance_private (self);
     g_auto (GStrv) extensions = NULL;
+
     if (!priv->shell_settings || (value == priv->extension_enabled))
         return;
 
@@ -804,7 +709,7 @@ g_paste_settings_set_extension_enabled (GPasteSettings *self,
     if (value)
     {
         extensions = g_realloc (extensions, (nb + 2) * sizeof (gchar *));
-        extensions[nb] = g_strdup (EXTENSION_NAME);
+        extensions[nb] = g_strdup (G_PASTE_EXTENSION_NAME);
         extensions[nb+1] = NULL;
     }
     else
@@ -812,7 +717,7 @@ g_paste_settings_set_extension_enabled (GPasteSettings *self,
         gboolean found = FALSE;
         for (guint64 i = 0; i < nb; ++i)
         {
-            if (!found && !g_strcmp0 (extensions[i], EXTENSION_NAME))
+            if (!found && g_paste_str_equal (extensions[i], G_PASTE_EXTENSION_NAME))
             {
                 found = TRUE;
                 g_free (extensions[i]);
@@ -843,7 +748,6 @@ g_paste_settings_shell_settings_changed (GSettings   *settings G_GNUC_UNUSED,
                    G_PASTE_EXTENSION_ENABLED_SETTING,
                    NULL);
 }
-#endif
 
 static void
 g_paste_settings_rebind (GPasteSettings *self,
@@ -863,61 +767,61 @@ g_paste_settings_settings_changed (GSettings   *settings G_GNUC_UNUSED,
     GPasteSettings *self = G_PASTE_SETTINGS (user_data);
     GPasteSettingsPrivate *priv = g_paste_settings_get_instance_private (self);
 
-    if (!g_strcmp0 (key, G_PASTE_ELEMENT_SIZE_SETTING))
+    if (g_paste_str_equal (key, G_PASTE_ELEMENT_SIZE_SETTING))
         g_paste_settings_private_set_element_size_from_dconf (priv);
-    else if (!g_strcmp0 (key, G_PASTE_GROWING_LINES_SETTING))
+    else if (g_paste_str_equal (key, G_PASTE_GROWING_LINES_SETTING))
         g_paste_settings_private_set_growing_lines_from_dconf (priv);
-    else if (!g_strcmp0 (key, G_PASTE_HISTORY_NAME_SETTING))
+    else if (g_paste_str_equal (key, G_PASTE_HISTORY_NAME_SETTING))
         g_paste_settings_private_set_history_name_from_dconf (priv);
-    else if (!g_strcmp0 (key, G_PASTE_IMAGES_SUPPORT_SETTING))
+    else if (g_paste_str_equal (key, G_PASTE_IMAGES_SUPPORT_SETTING))
         g_paste_settings_private_set_images_support_from_dconf (priv);
-    else if (!g_strcmp0 (key, G_PASTE_LAUNCH_UI_SETTING))
+    else if (g_paste_str_equal (key, G_PASTE_LAUNCH_UI_SETTING))
     {
         g_paste_settings_private_set_launch_ui_from_dconf (priv);
         g_paste_settings_rebind (self, G_PASTE_LAUNCH_UI_SETTING);
     }
-    else if (!g_strcmp0 (key, G_PASTE_MAKE_PASSWORD_SETTING))
+    else if (g_paste_str_equal (key, G_PASTE_MAKE_PASSWORD_SETTING))
     {
         g_paste_settings_private_set_make_password_from_dconf (priv);
         g_paste_settings_rebind (self, G_PASTE_MAKE_PASSWORD_SETTING);
     }
-    else if (!g_strcmp0 (key, G_PASTE_MAX_DISPLAYED_HISTORY_SIZE_SETTING))
+    else if (g_paste_str_equal (key, G_PASTE_MAX_DISPLAYED_HISTORY_SIZE_SETTING))
         g_paste_settings_private_set_max_displayed_history_size_from_dconf (priv);
-    else if (!g_strcmp0 (key, G_PASTE_MAX_HISTORY_SIZE_SETTING))
+    else if (g_paste_str_equal (key, G_PASTE_MAX_HISTORY_SIZE_SETTING))
         g_paste_settings_private_set_max_history_size_from_dconf (priv);
-    else if (!g_strcmp0 (key, G_PASTE_MAX_MEMORY_USAGE_SETTING))
+    else if (g_paste_str_equal (key, G_PASTE_MAX_MEMORY_USAGE_SETTING))
         g_paste_settings_private_set_max_memory_usage_from_dconf (priv);
-    else if (!g_strcmp0 (key, G_PASTE_MAX_TEXT_ITEM_SIZE_SETTING))
+    else if (g_paste_str_equal (key, G_PASTE_MAX_TEXT_ITEM_SIZE_SETTING))
         g_paste_settings_private_set_max_text_item_size_from_dconf (priv);
-    else if (!g_strcmp0 (key, G_PASTE_MIN_TEXT_ITEM_SIZE_SETTING))
+    else if (g_paste_str_equal (key, G_PASTE_MIN_TEXT_ITEM_SIZE_SETTING))
         g_paste_settings_private_set_min_text_item_size_from_dconf (priv);
-    else if (!g_strcmp0 (key, G_PASTE_POP_SETTING))
+    else if (g_paste_str_equal (key, G_PASTE_POP_SETTING))
     {
         g_paste_settings_private_set_pop_from_dconf (priv);
         g_paste_settings_rebind (self, G_PASTE_POP_SETTING);
     }
-    else if (!g_strcmp0 (key, G_PASTE_PRIMARY_TO_HISTORY_SETTING ))
+    else if (g_paste_str_equal (key, G_PASTE_PRIMARY_TO_HISTORY_SETTING ))
         g_paste_settings_private_set_primary_to_history_from_dconf (priv);
-    else if (!g_strcmp0 (key, G_PASTE_SAVE_HISTORY_SETTING))
+    else if (g_paste_str_equal (key, G_PASTE_SAVE_HISTORY_SETTING))
         g_paste_settings_private_set_save_history_from_dconf (priv);
-    else if (!g_strcmp0 (key, G_PASTE_SHOW_HISTORY_SETTING))
+    else if (g_paste_str_equal (key, G_PASTE_SHOW_HISTORY_SETTING))
     {
         g_paste_settings_private_set_show_history_from_dconf (priv);
         g_paste_settings_rebind (self, G_PASTE_SHOW_HISTORY_SETTING);
     }
-    else if (!g_strcmp0 (key, G_PASTE_SYNC_CLIPBOARD_TO_PRIMARY_SETTING))
+    else if (g_paste_str_equal (key, G_PASTE_SYNC_CLIPBOARD_TO_PRIMARY_SETTING))
     {
         g_paste_settings_private_set_sync_clipboard_to_primary_from_dconf (priv);
         g_paste_settings_rebind (self, G_PASTE_SYNC_CLIPBOARD_TO_PRIMARY_SETTING);
     }
-    else if (!g_strcmp0 (key, G_PASTE_SYNC_PRIMARY_TO_CLIPBOARD_SETTING))
+    else if (g_paste_str_equal (key, G_PASTE_SYNC_PRIMARY_TO_CLIPBOARD_SETTING))
     {
         g_paste_settings_private_set_sync_primary_to_clipboard_from_dconf (priv);
         g_paste_settings_rebind (self, G_PASTE_SYNC_PRIMARY_TO_CLIPBOARD_SETTING);
     }
-    else if (!g_strcmp0 (key, G_PASTE_SYNCHRONIZE_CLIPBOARDS_SETTING))
+    else if (g_paste_str_equal (key, G_PASTE_SYNCHRONIZE_CLIPBOARDS_SETTING))
         g_paste_settings_private_set_synchronize_clipboards_from_dconf (priv);
-    else if (!g_strcmp0 (key, G_PASTE_TRACK_CHANGES_SETTING))
+    else if (g_paste_str_equal (key, G_PASTE_TRACK_CHANGES_SETTING))
     {
         g_paste_settings_private_set_track_changes_from_dconf (priv);
         g_signal_emit (self,
@@ -926,11 +830,11 @@ g_paste_settings_settings_changed (GSettings   *settings G_GNUC_UNUSED,
                        priv->track_changes,
                        NULL);
     }
-    else if (!g_strcmp0 (key, G_PASTE_TRACK_EXTENSION_STATE_SETTING))
+    else if (g_paste_str_equal (key, G_PASTE_TRACK_EXTENSION_STATE_SETTING))
         g_paste_settings_private_set_track_extension_state_from_dconf (priv);
-    else if (!g_strcmp0 (key, G_PASTE_TRIM_ITEMS_SETTING))
+    else if (g_paste_str_equal (key, G_PASTE_TRIM_ITEMS_SETTING))
         g_paste_settings_private_set_trim_items_from_dconf (priv);
-    else if (!g_strcmp0 (key, G_PASTE_UPLOAD_SETTING))
+    else if (g_paste_str_equal (key, G_PASTE_UPLOAD_SETTING))
     {
         g_paste_settings_private_set_upload_from_dconf (priv);
         g_paste_settings_rebind (self, G_PASTE_UPLOAD_SETTING);
@@ -947,24 +851,21 @@ g_paste_settings_settings_changed (GSettings   *settings G_GNUC_UNUSED,
 static void
 g_paste_settings_dispose (GObject *object)
 {
-    GPasteSettingsPrivate *priv = g_paste_settings_get_instance_private (G_PASTE_SETTINGS (object));
+    const GPasteSettingsPrivate *priv = _g_paste_settings_get_instance_private (G_PASTE_SETTINGS (object));
     GSettings *settings = priv->settings;
+    GSettings *shell_settings = priv->shell_settings;
 
     if (settings)
     {
-        g_signal_handler_disconnect (settings, priv->changed_signal);
+        g_signal_handler_disconnect (settings, priv->c_signals[C_CHANGED]);
         g_clear_object (&priv->settings);
     }
 
-#ifdef ENABLE_EXTENSION
-    GSettings *shell_settings = priv->shell_settings;
-
     if (shell_settings)
     {
-        g_signal_handler_disconnect (shell_settings, priv->shell_changed_signal);
+        g_signal_handler_disconnect (shell_settings, priv->c_signals[C_SHELL_CHANGED]);
         g_clear_object (&priv->shell_settings);
     }
-#endif
 
     G_OBJECT_CLASS (g_paste_settings_parent_class)->dispose (object);
 }
@@ -972,7 +873,7 @@ g_paste_settings_dispose (GObject *object)
 static void
 g_paste_settings_finalize (GObject *object)
 {
-    GPasteSettingsPrivate *priv = g_paste_settings_get_instance_private (G_PASTE_SETTINGS (object));
+    const GPasteSettingsPrivate *priv = _g_paste_settings_get_instance_private (G_PASTE_SETTINGS (object));
 
     g_free (priv->history_name);
     g_free (priv->launch_ui);
@@ -1050,10 +951,10 @@ g_paste_settings_init (GPasteSettings *self)
     priv->sync_primary_to_clipboard = NULL;
     priv->upload = NULL;
 
-    priv->changed_signal = g_signal_connect (settings,
-                                             "changed",
-                                             G_CALLBACK (g_paste_settings_settings_changed),
-                                             self);
+    priv->c_signals[C_CHANGED] = g_signal_connect (settings,
+                                                   "changed",
+                                                   G_CALLBACK (g_paste_settings_settings_changed),
+                                                   self);
 
     g_paste_settings_private_set_element_size_from_dconf (priv);
     g_paste_settings_private_set_growing_lines_from_dconf (priv);
@@ -1081,25 +982,17 @@ g_paste_settings_init (GPasteSettings *self)
     priv->shell_settings = NULL;
     priv->extension_enabled = FALSE;
 
-#ifdef ENABLE_EXTENSION
-    GSettingsSchemaSource *source = g_settings_schema_source_get_default ();
-    if (!source)
-        return;
+    if (g_paste_util_has_gnome_shell ())
+    {
+        priv->shell_settings = g_settings_new (G_PASTE_SHELL_SETTINGS_NAME);
 
-    g_autoptr (GSettingsSchema) schema = g_settings_schema_source_lookup (source, G_PASTE_SHELL_SETTINGS_NAME, TRUE);
+        priv->c_signals[C_SHELL_CHANGED] = g_signal_connect (priv->shell_settings,
+                                                             "changed::" G_PASTE_SHELL_ENABLED_EXTENSIONS_SETTING,
+                                                             G_CALLBACK (g_paste_settings_shell_settings_changed),
+                                                             self);
 
-    if (!schema)
-        return;
-
-    priv->shell_settings = g_settings_new (G_PASTE_SHELL_SETTINGS_NAME);
-
-    priv->shell_changed_signal = g_signal_connect (priv->shell_settings,
-                                                  "changed::" G_PASTE_SHELL_ENABLED_EXTENSIONS_SETTING,
-                                                  G_CALLBACK (g_paste_settings_shell_settings_changed),
-                                                  self);
-
-    g_paste_settings_private_set_extension_enabled_from_dconf (priv);
-#endif
+        g_paste_settings_private_set_extension_enabled_from_dconf (priv);
+    }
 }
 
 /**
