@@ -1,22 +1,10 @@
 /*
- *      This file is part of GPaste.
+ * This file is part of GPaste.
  *
- *      Copyright 2015 Marc-Antoine Perennou <Marc-Antoine@Perennou.com>
- *
- *      GPaste is free software: you can redistribute it and/or modify
- *      it under the terms of the GNU General Public License as published by
- *      the Free Software Foundation, either version 3 of the License, or
- *      (at your option) any later version.
- *
- *      GPaste is distributed in the hope that it will be useful,
- *      but WITHOUT ANY WARRANTY; without even the implied warranty of
- *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *      GNU General Public License for more details.
- *
- *      You should have received a copy of the GNU General Public License
- *      along with GPaste.  If not, see <http://www.gnu.org/licenses/>.
+ * Copyright (c) 2010-2016, Marc-Antoine Perennou <Marc-Antoine@Perennou.com>
  */
 
+#include <gpaste-gsettings-keys.h>
 #include <gpaste-macros.h>
 #include <gpaste-util.h>
 
@@ -26,8 +14,6 @@
  * @msg: the message to display
  *
  * Show GPaste about dialog
- *
- * Returns:
  */
 G_PASTE_VISIBLE gboolean
 g_paste_util_confirm_dialog (GtkWindow   *parent,
@@ -99,8 +85,6 @@ g_paste_util_spawn_on_proxy_ready (GObject      *source_object G_GNUC_UNUSED,
  * @app: the GPaste app to spawn
  *
  * spawn a GPaste app
- *
- * Returns:
  */
 G_PASTE_VISIBLE void
 g_paste_util_spawn (const gchar *app)
@@ -213,8 +197,6 @@ g_paste_util_activate_ui_on_proxy_ready (GObject      *source_object G_GNUC_UNUS
  * @arg: (nullable): the action argument
  *
  * Activate an action on a GPaste app
- *
- * Returns:
  */
 G_PASTE_VISIBLE void
 g_paste_util_activate_ui (const gchar *action,
@@ -331,7 +313,7 @@ g_paste_util_replace (const gchar *text,
 G_PASTE_VISIBLE gchar *
 g_paste_util_compute_checksum (GdkPixbuf *image)
 {
-    if (!image || !GDK_IS_PIXBUF (image))
+    if (!image)
         return NULL;
 
     guint32 length;
@@ -366,13 +348,31 @@ g_paste_util_has_unity (void)
     return g_file_test (PKGLIBEXECDIR "/gpaste-app-indicator", G_FILE_TEST_IS_EXECUTABLE);
 }
 
+/**
+ * g_paste_util_has_gnome_shell:
+ *
+ * Check whether gnome-shell is installed or not
+ *
+ * Returns: %TRUE if gnome-shell is installed
+ */
+G_PASTE_VISIBLE gboolean
+g_paste_util_has_gnome_shell (void)
+{
+    GSettingsSchemaSource *source = g_settings_schema_source_get_default ();
+
+    if (!source)
+        return FALSE;
+
+    g_autoptr (GSettingsSchema) schema = g_settings_schema_source_lookup (source, G_PASTE_SHELL_SETTINGS_NAME, TRUE);
+
+    return !!schema;
+}
+
 /**
  * g_paste_util_show_win:
  * @application: a #GtkApplication
  *
  * Present the application's window to user
- *
- * Returns:
  */
 G_PASTE_VISIBLE void
 g_paste_util_show_win (GApplication *application)
@@ -431,3 +431,55 @@ g_paste_util_get_dbus_au_result (GVariant *variant,
 
     return ret;
 }
+
+static gchar *
+g_paste_util_get_runtime_dir (const gchar *component)
+{
+    return g_strdup_printf ("%s/" PACKAGE_NAME "/%s", g_get_user_runtime_dir (), component);
+}
+
+/**
+ * g_paste_util_write_pid_file:
+ * @component: The component we're handling
+ *
+ * Write the pid file
+ */
+G_PASTE_VISIBLE void
+g_paste_util_write_pid_file (const gchar *component)
+{
+    g_autofree gchar *dir = g_paste_util_get_runtime_dir (component);
+
+    g_mkdir_with_parents (dir, 0700);
+
+#ifdef G_OS_UNIX
+    g_autofree gchar *pidfile = g_strdup_printf ("%s/pid", dir);
+    g_autofree gchar *contents = g_strdup_printf ("%d", getpid ());
+
+    g_file_set_contents (pidfile, contents, -1, NULL);
+#endif
+}
+
+/**
+ * g_paste_util_read_pid_file:
+ * @component: The component we're handling
+ *
+ * Read the pid file
+ *
+ * Returns: the pid
+ */
+G_PASTE_VISIBLE GPid
+g_paste_util_read_pid_file (const gchar *component)
+{
+#ifdef G_OS_UNIX
+    g_autofree gchar *dir = g_paste_util_get_runtime_dir (component);
+    g_autofree gchar *pidfile = g_strdup_printf ("%s/pid", dir);
+    g_autofree gchar *contents = NULL;
+
+    if (!g_file_get_contents (pidfile, &contents, NULL, NULL))
+        return (GPid) -1;
+
+    return (GPid) g_ascii_strtoll (contents, NULL, 0);
+#else
+    return (GPid) -1;
+#endif
+}
