@@ -1,20 +1,7 @@
 /*
- *      This file is part of GPaste.
+ * This file is part of GPaste.
  *
- *      Copyright 2013-2015 Marc-Antoine Perennou <Marc-Antoine@Perennou.com>
- *
- *      GPaste is free software: you can redistribute it and/or modify
- *      it under the terms of the GNU General Public License as published by
- *      the Free Software Foundation, either version 3 of the License, or
- *      (at your option) any later version.
- *
- *      GPaste is distributed in the hope that it will be useful,
- *      but WITHOUT ANY WARRANTY; without even the implied warranty of
- *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *      GNU General Public License for more details.
- *
- *      You should have received a copy of the GNU General Public License
- *      along with GPaste.  If not, see <http://www.gnu.org/licenses/>.
+ * Copyright (c) 2010-2016, Marc-Antoine Perennou <Marc-Antoine@Perennou.com>
  */
 
 #include <gpaste-gnome-shell-client.h>
@@ -56,7 +43,7 @@ kill_loop (gpointer user_data)
 gint
 main (gint argc, gchar *argv[])
 {
-    if (argc != 2 || g_strcmp0 (argv[1], "--dont-skip"))
+    if (argc != 2 || !g_paste_str_equal (argv[1], "--dont-skip"))
         return EXIT_TEST_SKIP;
 
     g_setenv ("G_MESSAGES_DEBUG", "all", TRUE);
